package br.gov.mj.side.entidades.entidade;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;

import org.hibernate.annotations.Parameter;
import org.hibernate.annotations.Type;

import br.gov.mj.apoio.entidades.Municipio;
import br.gov.mj.apoio.entidades.TipoEndereco;
import br.gov.mj.apoio.entidades.TipoEntidade;
import br.gov.mj.infra.negocio.persistencia.BaseEntity;
import br.gov.mj.side.entidades.enums.EnumPersonalidadeJuridica;
import br.gov.mj.side.entidades.enums.EnumStatusEntidade;

@Entity
@Table(name = "tb_ent_entidade", schema = "side")
public class Entidade extends BaseEntity<Long> {
    private static final long serialVersionUID = 1L;

    @Id
    @Column(name = "ent_id_entidade")
    @SequenceGenerator(name = "tb_ent_entidade_generator", sequenceName = "side.seq_tb_ent_entidade", allocationSize = 1)
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "tb_ent_entidade_generator")
    @NotNull
    private Long id;

    @Enumerated(EnumType.STRING)
    @Type(type = "br.gov.mj.infra.negocio.persistencia.BaseEnumType", parameters = @Parameter(value = "br.gov.mj.side.entidades.enums.EnumStatusEntidade", name = "enumClassName"))
    @Column(name = "ent_st_status_entidade")
    @NotNull
    private EnumStatusEntidade statusEntidade;

    @Column(name = "ent_nu_numero_cnpj")
    @NotNull
    private String numeroCnpj;

    @ManyToOne
    @JoinColumn(name = "ent_fk_ten_id_tipo_entidade")
    @NotNull
    private TipoEntidade tipoEntidade;

    @Column(name = "ent_no_nome_entidade")
    @NotNull
    private String nomeEntidade;

    @Enumerated(EnumType.STRING)
    @Type(type = "br.gov.mj.infra.negocio.persistencia.BaseEnumType", parameters = @Parameter(value = "br.gov.mj.side.entidades.enums.EnumPersonalidadeJuridica", name = "enumClassName"))
    @Column(name = "ent_tp_tipo_natureza_juridica")
    @NotNull
    private EnumPersonalidadeJuridica personalidadeJuridica;

    @ManyToOne
    @JoinColumn(name = "ent_fk_mun_id_municipio")
    @NotNull
    private Municipio municipio;

    @ManyToOne
    @JoinColumn(name = "ent_fk_ted_id_tipo_endereco")
    @NotNull
    private TipoEndereco tipoEndereco;

    @Column(name = "ent_ds_descricao_endereco")
    @NotNull
    private String descricaoEndereco;

    @Column(name = "ent_nu_numero_endereco")
    @NotNull
    private String numeroEndereco;

    @Column(name = "ent_ds_descricao_complemento")
    private String complementoEndereco;

    @Column(name = "ent_ds_descricao_bairro")
    @NotNull
    private String bairro;

    @Column(name = "ent_nu_numero_cep")
    @NotNull
    private String numeroCep;

    @Column(name = "ent_nu_numero_telefone")
    @NotNull
    private String numeroTelefone;

    @Column(name = "ent_nu_numero_fone_fax")
    private String numeroFoneFax;

    @Column(name = "ent_ds_email")
    @NotNull
    private String email;

    @Column(name = "ent_no_usuario_cadastro")
    @NotNull
    private String usuarioCadastro;

    @Column(name = "ent_no_usuario_alteracao")
    private String usuarioAlteracao;

    @Type(type = "org.jadira.usertype.dateandtime.threeten.PersistentLocalDateTime")
    @Column(name = "ent_dt_data_cadastro")
    @NotNull
    private LocalDateTime dataCadastro;

    @Type(type = "org.jadira.usertype.dateandtime.threeten.PersistentLocalDateTime")
    @Column(name = "ent_dt_data_alteracao")
    private LocalDateTime dataAlteracao;

    @Column(name = "ent_nu_numero_processo_sei")
    @NotNull
    private String numeroProcessoSEI;

    @OneToMany(mappedBy = "entidade", cascade = { CascadeType.ALL }, orphanRemoval = true)
    private List<EntidadeAnexo> anexos = new ArrayList<EntidadeAnexo>();

    @OneToMany(mappedBy = "entidade", cascade = { CascadeType.ALL }, orphanRemoval = true)
    private List<PessoaEntidade> pessoas = new ArrayList<PessoaEntidade>();

    @OneToMany(mappedBy = "entidade", cascade = { CascadeType.ALL }, orphanRemoval = true)
    private List<LocalEntregaEntidade> locaisEntregaEntidade = new ArrayList<LocalEntregaEntidade>();

    public Entidade() {
    }

    @Override
    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public EnumStatusEntidade getStatusEntidade() {
        return statusEntidade;
    }

    public void setStatusEntidade(EnumStatusEntidade statusEntidade) {
        this.statusEntidade = statusEntidade;
    }

    public String getNumeroCnpj() {
        return numeroCnpj;
    }

    public void setNumeroCnpj(String numeroCnpj) {
        this.numeroCnpj = numeroCnpj;
    }

    public TipoEntidade getTipoEntidade() {
        return tipoEntidade;
    }

    public void setTipoEntidade(TipoEntidade tipoEntidade) {
        this.tipoEntidade = tipoEntidade;
    }

    public String getNomeEntidade() {
        return nomeEntidade;
    }

    public void setNomeEntidade(String nomeEntidade) {
        this.nomeEntidade = nomeEntidade;
    }

    public EnumPersonalidadeJuridica getPersonalidadeJuridica() {
        return personalidadeJuridica;
    }

    public void setPersonalidadeJuridica(EnumPersonalidadeJuridica personalidadeJuridica) {
        this.personalidadeJuridica = personalidadeJuridica;
    }

    public Municipio getMunicipio() {
        return municipio;
    }

    public void setMunicipio(Municipio municipio) {
        this.municipio = municipio;
    }

    public TipoEndereco getTipoEndereco() {
        return tipoEndereco;
    }

    public void setTipoEndereco(TipoEndereco tipoEndereco) {
        this.tipoEndereco = tipoEndereco;
    }

    public String getDescricaoEndereco() {
        return descricaoEndereco;
    }

    public void setDescricaoEndereco(String descricaoEndereco) {
        this.descricaoEndereco = descricaoEndereco;
    }

    public String getNumeroEndereco() {
        return numeroEndereco;
    }

    public void setNumeroEndereco(String numeroEndereco) {
        this.numeroEndereco = numeroEndereco;
    }

    public String getComplementoEndereco() {
        return complementoEndereco;
    }

    public void setComplementoEndereco(String complementoEndereco) {
        this.complementoEndereco = complementoEndereco;
    }

    public String getBairro() {
        return bairro;
    }

    public void setBairro(String bairro) {
        this.bairro = bairro;
    }

    public String getNumeroCep() {
        return numeroCep;
    }

    public void setNumeroCep(String numeroCep) {
        this.numeroCep = numeroCep;
    }

    public String getNumeroTelefone() {
        return numeroTelefone;
    }

    public void setNumeroTelefone(String numeroTelefone) {
        this.numeroTelefone = numeroTelefone;
    }

    public String getNumeroFoneFax() {
        return numeroFoneFax;
    }

    public void setNumeroFoneFax(String numeroFoneFax) {
        this.numeroFoneFax = numeroFoneFax;
    }

    public String getEmail() {
        return email;
    }

    public void setEmail(String email) {
        this.email = email;
    }

    public String getUsuarioCadastro() {
        return usuarioCadastro;
    }

    public void setUsuarioCadastro(String usuarioCadastro) {
        this.usuarioCadastro = usuarioCadastro;
    }

    public String getUsuarioAlteracao() {
        return usuarioAlteracao;
    }

    public void setUsuarioAlteracao(String usuarioAlteracao) {
        this.usuarioAlteracao = usuarioAlteracao;
    }

    public LocalDateTime getDataCadastro() {
        return dataCadastro;
    }

    public void setDataCadastro(LocalDateTime dataCadastro) {
        this.dataCadastro = dataCadastro;
    }

    public LocalDateTime getDataAlteracao() {
        return dataAlteracao;
    }

    public void setDataAlteracao(LocalDateTime dataAlteracao) {
        this.dataAlteracao = dataAlteracao;
    }

    public List<EntidadeAnexo> getAnexos() {
        return anexos;
    }

    public void setAnexos(List<EntidadeAnexo> anexos) {
        this.anexos = anexos;
    }

    public List<PessoaEntidade> getPessoas() {
        return pessoas;
    }

    public void setPessoas(List<PessoaEntidade> pessoas) {
        this.pessoas = pessoas;
    }

    public String getNumeroProcessoSEI() {
        return numeroProcessoSEI;
    }

    public void setNumeroProcessoSEI(String numeroProcessoSEI) {
        this.numeroProcessoSEI = numeroProcessoSEI;
    }

    public String getEnderecoCompleto() {
        return getTipoEndereco().getDescricaoTipoEndereco() + " " + getDescricaoEndereco() + " " + getComplementoEndereco() + " - Nº: " + getNumeroEndereco() + " - Bairro: " + getBairro() + " - Cidade: " + getMunicipio().getNomeMunicipio() + " - " + getMunicipio().getUf().getSiglaUf() + " - CEP:"
                + getNumeroCep();
    }

    public List<LocalEntregaEntidade> getLocaisEntregaEntidade() {
        return locaisEntregaEntidade;
    }

    public void setLocaisEntregaEntidade(List<LocalEntregaEntidade> locaisEntregaEntidade) {
        this.locaisEntregaEntidade = locaisEntregaEntidade;
    }

}