package br.gov.mj.side.web.dao;

import java.util.ArrayList;
import java.util.List;

import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import br.gov.mj.side.entidades.AnexoBem;

@Stateless
@TransactionAttribute(TransactionAttributeType.MANDATORY)
public class AnexoBemDAO {

    @Inject
    private EntityManager em;

    public List<AnexoBem> buscarPeloIdBem(Long idBem) {
        CriteriaBuilder criteriaBuilder = em.getCriteriaBuilder();
        CriteriaQuery<AnexoBem> criteriaQuery = criteriaBuilder.createQuery(AnexoBem.class);
        Root<AnexoBem> root = criteriaQuery.from(AnexoBem.class);

        Predicate[] predicates = extractPredicates(idBem, criteriaBuilder, root);
        criteriaQuery.select(criteriaQuery.getSelection()).where(predicates);

        criteriaQuery.orderBy(criteriaBuilder.asc(root.get("id")));

        TypedQuery<AnexoBem> query = em.createQuery(criteriaQuery);
        return query.getResultList();
    }

    public AnexoBem buscarPeloId(Long id) {
        return em.find(AnexoBem.class, id);
    }

    public Long countPeloIdBem(Long idBem) {
        CriteriaBuilder criteriaBuilder = em.getCriteriaBuilder();
        CriteriaQuery<Long> criteriaQuery = criteriaBuilder.createQuery(Long.class);
        Root<AnexoBem> root = criteriaQuery.from(AnexoBem.class);

        criteriaQuery.select(criteriaBuilder.count(root));
        Predicate[] predicates = extractPredicates(idBem, criteriaBuilder, root);
        criteriaQuery.where(predicates);

        return em.createQuery(criteriaQuery).getSingleResult();
    }

    private Predicate[] extractPredicates(Long idBem, CriteriaBuilder criteriaBuilder, Root<?> root) {
        List<Predicate> predicates = new ArrayList<>();
        if (idBem != null) {
            predicates.add(criteriaBuilder.equal(root.get("bem").get("id"), idBem));
        }
        return predicates.toArray(new Predicate[] {});
    }

}
