package br.gov.mj.side.web.dao;

import java.util.ArrayList;
import java.util.List;

import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import br.gov.mj.side.entidades.entidade.EntidadeAnexo;

@Stateless
@TransactionAttribute(TransactionAttributeType.MANDATORY)
public class AnexoEntidadeDAO {

    @Inject
    private EntityManager em;

    public List<EntidadeAnexo> buscarPeloIdEntidade(Long idEntidade) {
        CriteriaBuilder criteriaBuilder = em.getCriteriaBuilder();
        CriteriaQuery<EntidadeAnexo> criteriaQuery = criteriaBuilder.createQuery(EntidadeAnexo.class);
        Root<EntidadeAnexo> root = criteriaQuery.from(EntidadeAnexo.class);

        Predicate[] predicates = extractPredicates(idEntidade, criteriaBuilder, root);
        criteriaQuery.select(criteriaQuery.getSelection()).where(predicates);

        criteriaQuery.orderBy(criteriaBuilder.asc(root.get("id")));

        TypedQuery<EntidadeAnexo> query = em.createQuery(criteriaQuery);
        return query.getResultList();
    }

    public EntidadeAnexo buscarPeloId(Long id) {
        return em.find(EntidadeAnexo.class, id);
    }

    public Long countPeloIdEntidade(Long idEntidade) {
        CriteriaBuilder criteriaBuilder = em.getCriteriaBuilder();
        CriteriaQuery<Long> criteriaQuery = criteriaBuilder.createQuery(Long.class);
        Root<EntidadeAnexo> root = criteriaQuery.from(EntidadeAnexo.class);

        criteriaQuery.select(criteriaBuilder.count(root));
        Predicate[] predicates = extractPredicates(idEntidade, criteriaBuilder, root);
        criteriaQuery.where(predicates);

        return em.createQuery(criteriaQuery).getSingleResult();
    }

    private Predicate[] extractPredicates(Long idEntidade, CriteriaBuilder criteriaBuilder, Root<?> root) {
        List<Predicate> predicates = new ArrayList<>();
        if (idEntidade != null) {
            predicates.add(criteriaBuilder.equal(root.get("entidade").get("id"), idEntidade));
        }
        return predicates.toArray(new Predicate[] {});
    }

}
