package br.gov.mj.side.web.dao;

import java.util.ArrayList;
import java.util.List;

import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import br.gov.mj.side.entidades.programa.ProgramaAnexo;

@Stateless
@TransactionAttribute(TransactionAttributeType.MANDATORY)
public class AnexoProgramaDAO {

    @Inject
    private EntityManager em;

    public List<ProgramaAnexo> buscarPeloIdPrograma(Long idPrograma) {
        CriteriaBuilder criteriaBuilder = em.getCriteriaBuilder();
        CriteriaQuery<ProgramaAnexo> criteriaQuery = criteriaBuilder.createQuery(ProgramaAnexo.class);
        Root<ProgramaAnexo> root = criteriaQuery.from(ProgramaAnexo.class);

        Predicate[] predicates = extractPredicates(idPrograma, criteriaBuilder, root);
        criteriaQuery.select(criteriaQuery.getSelection()).where(predicates);

        criteriaQuery.orderBy(criteriaBuilder.asc(root.get("id")));

        TypedQuery<ProgramaAnexo> query = em.createQuery(criteriaQuery);
        return query.getResultList();
    }

    public ProgramaAnexo buscarPeloId(Long id) {
        return em.find(ProgramaAnexo.class, id);
    }

    public Long countPeloIdPrograma(Long idPrograma) {
        CriteriaBuilder criteriaBuilder = em.getCriteriaBuilder();
        CriteriaQuery<Long> criteriaQuery = criteriaBuilder.createQuery(Long.class);
        Root<ProgramaAnexo> root = criteriaQuery.from(ProgramaAnexo.class);

        criteriaQuery.select(criteriaBuilder.count(root));
        Predicate[] predicates = extractPredicates(idPrograma, criteriaBuilder, root);
        criteriaQuery.where(predicates);

        return em.createQuery(criteriaQuery).getSingleResult();
    }

    private Predicate[] extractPredicates(Long idPrograma, CriteriaBuilder criteriaBuilder, Root<?> root) {
        List<Predicate> predicates = new ArrayList<>();
        if (idPrograma != null) {
            predicates.add(criteriaBuilder.equal(root.get("programa").get("id"), idPrograma));
        }
        return predicates.toArray(new Predicate[] {});
    }

}
