package br.gov.mj.side.web.service;

import java.util.List;

import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.inject.Inject;

import org.apache.commons.lang3.StringUtils;

import br.gov.mj.infra.negocio.exception.BusinessException;
import br.gov.mj.infra.negocio.persistencia.EnumOrder;
import br.gov.mj.infra.negocio.persistencia.IGenericPersister;
import br.gov.mj.side.entidades.enums.EnumStatusLicitacao;
import br.gov.mj.side.entidades.programa.licitacao.AgrupamentoLicitacao;
import br.gov.mj.side.entidades.programa.licitacao.BemUf;
import br.gov.mj.side.entidades.programa.licitacao.LicitacaoPrograma;
import br.gov.mj.side.entidades.programa.licitacao.SelecaoItem;
import br.gov.mj.side.web.dao.LicitacaoProgramaDAO;
import br.gov.mj.side.web.dto.PesquisaLicitacaoDto;

@Stateless
@TransactionAttribute(TransactionAttributeType.REQUIRED)
public class LicitacaoProgramaService {

    @Inject
    private IGenericPersister genericPersister;

    @Inject
    private LicitacaoProgramaDAO licitacaoProgramaDAO;

    public List<AgrupamentoLicitacao> buscarAgrupamentoLicitacao(LicitacaoPrograma licitacaoPrograma) {
        if (licitacaoPrograma == null || licitacaoPrograma.getId() == null) {
            throw new IllegalArgumentException("Parâmetro id não pode ser null");
        }
        return licitacaoProgramaDAO.buscarAgrupamentoLicitacao(licitacaoPrograma.getId());
    }

    public List<SelecaoItem> buscarSelecaoItem(AgrupamentoLicitacao agrupamentoLicitacao) {
        if (agrupamentoLicitacao == null || agrupamentoLicitacao.getId() == null) {
            throw new IllegalArgumentException("Parâmetro id não pode ser null");
        }
        return licitacaoProgramaDAO.buscarSelecaoItem(agrupamentoLicitacao.getId());
    }

    public List<BemUf> buscarBemUf(SelecaoItem selecaoItem) {
        if (selecaoItem == null || selecaoItem.getId() == null) {
            throw new IllegalArgumentException("Parâmetro id não pode ser null");
        }
        return licitacaoProgramaDAO.buscarBemUf(selecaoItem.getId());
    }

    public List<LicitacaoPrograma> buscarPaginado(PesquisaLicitacaoDto pesquisaLicitacaoDto, int first, int size, EnumOrder order, String propertyOrder) {

        if (pesquisaLicitacaoDto == null) {
            throw new IllegalArgumentException("Parâmetro pesquisaLicitacaoDto não pode ser null");
        }

        return licitacaoProgramaDAO.buscarPaginado(pesquisaLicitacaoDto, first, size, order, propertyOrder);
    }

    public List<LicitacaoPrograma> buscarSemPaginacao(PesquisaLicitacaoDto pesquisaLicitacaoDto) {

        if (pesquisaLicitacaoDto == null) {
            throw new IllegalArgumentException("Parâmetro pesquisaLicitacaoDto não pode ser null");
        }

        return licitacaoProgramaDAO.buscarSemPaginacao(pesquisaLicitacaoDto);
    }

    public List<LicitacaoPrograma> buscarSemPaginacaoOrdenado(PesquisaLicitacaoDto pesquisaLicitacaoDto, EnumOrder order, String propertyOrder) {

        if (pesquisaLicitacaoDto == null) {
            throw new IllegalArgumentException("Parâmetro pesquisaLicitacaoDto não pode ser null");
        }

        return licitacaoProgramaDAO.buscarSemPaginacaoOrdenado(pesquisaLicitacaoDto, order, propertyOrder);
    }

    public LicitacaoPrograma buscarPeloId(Long id) {

        if (id == null) {
            throw new IllegalArgumentException("Parâmetro id não pode ser null");
        }

        BusinessException ex = new BusinessException();

        /* Verificar se existe Inscricao com esse id cadastrado */

        LicitacaoPrograma l = licitacaoProgramaDAO.buscarPeloId(id);

        if (l == null) {
            /* msg cadastrada no arquivo de propriedades */
            ex.addErrorMessage("MN047", id);
            throw ex;
        }
        return l;
    }

    public LicitacaoPrograma incluirAlterar(LicitacaoPrograma licitacaoPrograma, String usuarioLogado) {

        LicitacaoPrograma licitacaoProgramaRetorno = null;

        validarObjetoNulo(licitacaoPrograma);

        BusinessException ex = new BusinessException();

        if (StringUtils.isBlank(usuarioLogado)) {
            ex.addErrorMessage("Usuario logado obrigatório.");
            throw ex;
        }

        if (licitacaoPrograma.getPrograma() == null || licitacaoPrograma.getPrograma().getId() == null) {
            ex.addErrorMessage("Escolha do Programa Obrigatória.");
            throw ex;
        }

        licitacaoPrograma.setStatusLicitacao(EnumStatusLicitacao.EM_ELABORACAO);

        if (licitacaoPrograma.getId() == null) {
            licitacaoProgramaRetorno = licitacaoProgramaDAO.incluir(licitacaoPrograma, usuarioLogado);
        } else {
            licitacaoProgramaRetorno = licitacaoProgramaDAO.alterar(licitacaoPrograma, usuarioLogado);
        }
        return licitacaoProgramaRetorno;
    }

    public void excluir(Long id) {

        if (id == null) {
            throw new IllegalArgumentException("Parâmetro id não pode ser null");
        }

        BusinessException ex = new BusinessException();

        /* Verificar se existe Licitacao com esse id cadastrado */

        LicitacaoPrograma l = genericPersister.findByUniqueProperty(LicitacaoPrograma.class, "id", id);
        if (l == null) {
            /* msg cadastrada no arquivo de propriedades */
            ex.addErrorMessage("MN047", id);
            throw ex;
        }

        licitacaoProgramaDAO.excluir(l);

    }

    private void validarObjetoNulo(LicitacaoPrograma licitacaoPrograma) {
        if (licitacaoPrograma == null) {
            throw new IllegalArgumentException("Parâmetro programa não pode ser null");
        }
    }

}
