package br.gov.mj.side.web.service;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.Resource;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.inject.Inject;
import javax.mail.Session;

import br.gov.mj.seg.entidades.Usuario;
import br.gov.mj.side.entidades.entidade.Entidade;
import br.gov.mj.side.entidades.entidade.Pessoa;
import br.gov.mj.side.entidades.entidade.PessoaEntidade;
import br.gov.mj.side.entidades.enums.EnumStatusPessoa;
import br.gov.mj.side.entidades.enums.EnumTipoLista;
import br.gov.mj.side.entidades.programa.Programa;
import br.gov.mj.side.entidades.programa.ProgramaHistoricoPublicizacao;
import br.gov.mj.side.entidades.programa.inscricao.InscricaoPrograma;
import br.gov.mj.side.web.dao.InscricaoProgramaDAO;
import br.gov.mj.side.web.util.DataUtil;
import br.gov.mj.side.web.util.MailSender;
import br.gov.mj.side.web.view.enums.EnumSelecao;

@Stateless
@TransactionAttribute(TransactionAttributeType.REQUIRED)
public class MailService {

    private static final String ASSUNTO_SIDE_CADASTRO_ENTIDADE = "SIDE - Cadastro de Entidade";
    private static final String ASSUNTO_SIDE_ACESSO_SISTEMA = "SIDE - Acesso ao Sistema";
    private static String FROM = "side@mj.gov.br";

    @Resource(mappedName = "java:jboss/mail/mj-mail")
    private Session session;

    // @Inject
    // MailUtil mailUtil;

    @Inject
    private MailSender mailSender;

    @Inject
    private EntidadeService entidadeService;

    @Inject
    private InscricaoProgramaDAO inscricaoProgramaDAO;

    @Inject
    private PublicizacaoService publicizacaoService;

    public void enviarEmailPublicacaoLista(Programa programa, EnumSelecao selecao, EnumTipoLista tipoLista) {

        List<InscricaoPrograma> lista = inscricaoProgramaDAO.buscarInscricaoProgramaPeloPrograma(programa);
        String assunto = "SIDE - Lista de " + selecao.getDescricao() + " - " + tipoLista.getDescricao();
        ProgramaHistoricoPublicizacao ultimoHistorico = publicizacaoService.buscarUltimoProgramaHistoricoPublicizacao(programa);
        String dataInicioRecurso = "";
        String dataFimRecurso = "";
        String dataInicioLocalEntrega = "";
        String dataFimLocalEntrega = "";

        if (selecao.getValor().equals(EnumSelecao.PROPOSTAS_ELEGIVEIS.getValor())) {
            dataInicioRecurso = DataUtil.converteDataDeLocalDateParaString(ultimoHistorico.getDataInicialRecursoElegibilidade(), "dd/MM/yyyy");
            dataFimRecurso = DataUtil.converteDataDeLocalDateParaString(ultimoHistorico.getDataFinalRecursoElegibilidade(), "dd/MM/yyyy");
        } else if (selecao.getValor().equals(EnumSelecao.CLASSIFICACAO_PROSPOSTA.getValor())) {
            dataInicioRecurso = DataUtil.converteDataDeLocalDateParaString(ultimoHistorico.getDataInicialRecursoAvaliacao(), "dd/MM/yyyy");
            dataFimRecurso = DataUtil.converteDataDeLocalDateParaString(ultimoHistorico.getDataFinalRecursoAvaliacao(), "dd/MM/yyyy");
            dataInicioLocalEntrega = DataUtil.converteDataDeLocalDateParaString(ultimoHistorico.getDataInicialCadastroLocalEntrega(), "dd/MM/yyyy");
            dataFimLocalEntrega = DataUtil.converteDataDeLocalDateParaString(ultimoHistorico.getDataFinalCadastroLocalEntrega(), "dd/MM/yyyy");
        }

        for (InscricaoPrograma inscricaoPrograma : lista) {
            mailSender.send(inscricaoPrograma.getPessoaEntidade().getPessoa().getUsuario().getEmail(), assunto,
                    geraMensagemLista(inscricaoPrograma.getPessoaEntidade().getPessoa().getNomePessoa(), inscricaoPrograma.getPrograma().getNomePrograma(), dataInicioRecurso, dataFimRecurso, tipoLista, selecao, dataInicioLocalEntrega, dataFimLocalEntrega), true);
        }
    }

    public void enviarEmailEsquecimentoSenha(Usuario usuario, String linkAplicacao) {

        mailSender.send(usuario.getEmail(), ASSUNTO_SIDE_ACESSO_SISTEMA, geraMensagemResetSenhaSistema(usuario, linkAplicacao), true);
        // MailDTO mailDTO = new MailDTO();
        // mailDTO.setFrom(FROM);
        // mailDTO.addTo(usuario.getEmail());
        // mailDTO.setSubject(ASSUNTO_SIDE_ACESSO_SISTEMA);
        // mailDTO.setMessage(geraMensagemResetSenhaSistema(usuario,
        // linkAplicacao));
        // mailDTO.setType(MailEventType.PLAIN);
        //
        // try {
        // mailUtil.sendMail(session, mailDTO);
        // } catch (AddressException e) {
        // e.printStackTrace();
        // } catch (MessagingException e) {
        // e.printStackTrace();
        // }
    }

    public void enviarEmailCadastroNovaEntidadePorUsuarioInterno(Entidade entidadePersistida, Long maiorIdPessoaAtual, String linkAplicacao) {

        List<PessoaEntidade> pessoasRetorno = entidadeService.buscarPessoa(entidadePersistida);

        List<PessoaEntidade> listaPessoaFinal = new ArrayList<PessoaEntidade>();

        Pessoa entidadeTitular = pegarTitular(pessoasRetorno);

        for (PessoaEntidade objPessoa : pessoasRetorno) {
            if (objPessoa.getPessoa().isRepresentante() && objPessoa.getPessoa().getId() > maiorIdPessoaAtual) {
                mailSender.send(objPessoa.getPessoa().getUsuario().getEmail(), ASSUNTO_SIDE_ACESSO_SISTEMA, geraMensagemAcessoSistema(objPessoa.getPessoa(), entidadePersistida, linkAplicacao), true);
                mailSender.send(entidadeTitular.getEmail(), ASSUNTO_SIDE_CADASTRO_ENTIDADE, geraMensagemCadastroEntidade(entidadeTitular, entidadePersistida, objPessoa.getPessoa()), true);
                // MailDTO mailDTO1 = new MailDTO();
                // mailDTO1.setFrom(FROM);
                // mailDTO1.addTo(objPessoa.getPessoa().getUsuario().getEmail());
                // mailDTO1.setSubject(ASSUNTO_SIDE_ACESSO_SISTEMA);
                // mailDTO1.setMessage(geraMensagemAcessoSistema(objPessoa.getPessoa(),
                // entidadePersistida, linkAplicacao));
                // mailDTO1.setType(MailEventType.PLAIN);
                //
                // MailDTO mailDTO2 = new MailDTO();
                // mailDTO2.setFrom(FROM);
                // mailDTO2.addTo(entidadeTitular.getEmail());
                // mailDTO2.setSubject(ASSUNTO_SIDE_CADASTRO_ENTIDADE);
                // mailDTO2.setMessage(geraMensagemCadastroEntidade(entidadeTitular,
                // entidadePersistida, objPessoa.getPessoa()));
                // mailDTO2.setType(MailEventType.PLAIN);
                //
                // try {
                // mailUtil.sendMail(session, mailDTO1);
                // mailUtil.sendMail(session, mailDTO2);
                // } catch (AddressException e) {
                // e.printStackTrace();
                // } catch (MessagingException e) {
                // e.printStackTrace();
                // }

            }
            listaPessoaFinal.add(objPessoa);
        }
    }

    private String geraMensagemAcessoSistema(Pessoa pessoa, Entidade entidade, String linkAplicacao) {
        String novaLinha = "<br/>";

        String inicioNegrito = "<b>";
        String fimNegrito = "</b>";
        String link = inicioNegrito + "<a href=" + linkAplicacao + "?hash=" + pessoa.getUsuario().getHashEnvioTrocaSenha() + " target='_blank'>Definir sua senha para o SIDE</a>" + fimNegrito;

        StringBuffer sb = new StringBuffer();
        sb.append("Prezado(a) " + inicioNegrito + pessoa.getNomePessoa().toUpperCase() + fimNegrito + ",").append(novaLinha);
        sb.append("O seu cadastro da Entidade " + inicioNegrito + entidade.getNomeEntidade().toUpperCase() + fimNegrito + " foi efetuado com sucesso em nosso sistema.").append(novaLinha);
        sb.append("Seu login é :" + inicioNegrito + pessoa.getUsuario().getNumeroCpf() + fimNegrito).append(novaLinha).append(novaLinha);
        sb.append("Utilize o seguinte link para seu primeiro acesso:").append(novaLinha).append(novaLinha);
        sb.append(link).append(novaLinha).append(novaLinha);
        sb.append("Acesse o SIDE para cadastrar uma senha de acesso que o possibilitará de acessar nosso sistema e se inscrever em Programas que estejam disponibilizados.").append(novaLinha);
        sb.append("Importante:").append(novaLinha);
        sb.append("A senha de acesso ao cadastro é pessoal e intransferível e seu uso é restrito ao próprio Representante, não podendo cedê-la ou transferi-la a terceiros.").append(novaLinha);
        sb.append("Verifique regularmente a capacidade de sua caixa de mensagens, a fim de evitar o não recebimento de mensagens encaminhadas por nosso sistema. Não nos responsabilizaremos por mensagens que não venham a ser entregues devido a fatores dessa natureza.").append(novaLinha);
        return sb.toString();
    }

    private String geraMensagemCadastroEntidade(Pessoa entidadeTitular, Entidade entidade, Pessoa entidadeRepresentante) {

        String novaLinha = "<br/>";
        String inicioNegrito = "<b>";
        String fimNegrito = "</b>";
        StringBuffer sb = new StringBuffer();
        sb.append("Prezado(a) " + inicioNegrito + entidadeTitular.getNomePessoa().toUpperCase() + fimNegrito + ",").append(novaLinha);
        sb.append("Informamos que o cadastro da Entidade " + inicioNegrito + entidade.getNomeEntidade().toUpperCase() + fimNegrito + " foi efetuado com sucesso em nosso sistema.").append(novaLinha);
        sb.append("O acesso ao SIDE - Sistema de Doações e Equipagem será efetuado pelo Representante " + inicioNegrito + entidadeRepresentante.getNomePessoa().toUpperCase() + fimNegrito + ".").append(novaLinha);

        return sb.toString();
    }

    private String geraMensagemResetSenhaSistema(Usuario usuario, String linkAplicacao) {
        String novaLinha = "<br/>";
        String inicioNegrito = "<b>";
        String fimNegrito = "</b>";
        String link = inicioNegrito + "<a href=" + linkAplicacao + "?hash=" + usuario.getHashEnvioTrocaSenha() + " target='_blank'>Alterar sua senha para o SIDE</a>" + fimNegrito;
        StringBuffer sb = new StringBuffer();
        sb.append("Prezado(a) usuário,").append(novaLinha);
        sb.append("A sua solicitação de reset de senha foi efetuada com sucesso em nosso sistema.").append(novaLinha);
        sb.append("Seu login é :" + inicioNegrito + usuario.getNumeroCpf() + fimNegrito).append(novaLinha).append(novaLinha);
        sb.append("Utilize o seguinte link para alterar sua senha:").append(novaLinha).append(novaLinha);
        sb.append(link).append(novaLinha).append(novaLinha);
        sb.append("Acesse o SIDE para cadastrar uma senha de acesso que o possibilitará de acessar nosso sistema e se inscrever em Programas que estejam disponibilizados.").append(novaLinha);
        sb.append("Importante:").append(novaLinha);
        sb.append("A senha de acesso ao cadastro é pessoal e intransferível e seu uso é restrito ao próprio Representante, não podendo cedê-la ou transferi-la a terceiros.").append(novaLinha);
        sb.append("Verifique regularmente a capacidade de sua caixa de mensagens, a fim de evitar o não recebimento de mensagens encaminhadas por nosso sistema. Não nos responsabilizaremos por mensagens que não venham a ser entregues devido a fatores dessa natureza.").append(novaLinha);

        return sb.toString();
    }

    private String geraMensagemLista(String nomeRepresentante, String nomePrograma, String dataInicioRecurso, String dataFimRecurso, EnumTipoLista tipoLista, EnumSelecao selecao, String dataInicioLocalEntrega, String dataFimLocalEntrega) {
        String novaLinha = "<br/>";
        String inicioNegrito = "<b>";
        String fimNegrito = "</b>";
        StringBuffer sb = new StringBuffer();
        sb.append("Prezado(a) " + nomeRepresentante + ",").append(novaLinha).append(novaLinha);
        sb.append("Informamos que a Lista " + tipoLista.getDescricao() + " de " + selecao.getDescricao() + " para o programa " + nomePrograma + " já está disponível.").append(novaLinha);
        sb.append("Para visualizar a lista acesse em nosso sistema o referido programa.").append(novaLinha).append(novaLinha).append(novaLinha);
        if (EnumTipoLista.PRELIMINAR.equals(tipoLista)) {
            sb.append("O período para recorrer ao resultado desta fase é de " + inicioNegrito + dataInicioRecurso + fimNegrito + " a " + inicioNegrito + dataFimRecurso + fimNegrito + ".").append(novaLinha).append(novaLinha).append(novaLinha);
        }

        if (EnumSelecao.CLASSIFICACAO_PROSPOSTA.equals(selecao) && EnumTipoLista.DEFINITIVA.equals(tipoLista)) {
            sb.append(
                    "Para os classificados o período para indicar os locais de entrega dos itens é de " + inicioNegrito + dataInicioLocalEntrega + fimNegrito + " a " + inicioNegrito + dataFimLocalEntrega + fimNegrito
                            + ". Para realizar a indicação, acessar a inscrição do programa na aba \"Endereços para Entrega\".").append(novaLinha).append(novaLinha).append(novaLinha);
        }

        sb.append(inicioNegrito + "Importante:" + fimNegrito).append(novaLinha).append(novaLinha);
        sb.append("Verifique regularmente a capacidade de sua caixa de mensagens a fim de evitar o não recebimento de mensagens encaminhadas por nosso sistema. Não nos responsabilizaremos por mensagens que não venham a ser entregues devido a fatores dessa natureza.").append(novaLinha);
        return sb.toString();
    }

    @SuppressWarnings("unused")
    private Pessoa pegarRepresentante(Entidade entidade) {
        for (PessoaEntidade pessoa : entidade.getPessoas()) {
            if (pessoa.getPessoa().isRepresentante() && pessoa.getPessoa().getStatusPessoa().equals(EnumStatusPessoa.ATIVO)) {
                return pessoa.getPessoa();
            }
        }
        return null;
    }

    private Pessoa pegarTitular(List<PessoaEntidade> pessoas) {
        for (PessoaEntidade pessoa : pessoas) {
            if (pessoa.getPessoa().isTitular() && pessoa.getPessoa().getStatusPessoa().equals(EnumStatusPessoa.ATIVO)) {
                return pessoa.getPessoa();
            }
        }
        return null;
    }

}
