package br.gov.mj.side.web.view;

import java.util.List;

import javax.inject.Inject;

import org.apache.wicket.RestartResponseAtInterceptPageException;
import org.apache.wicket.authroles.authentication.AuthenticatedWebSession;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.form.PasswordTextField;
import org.apache.wicket.markup.html.form.StatelessForm;
import org.apache.wicket.markup.html.form.SubmitLink;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.request.Url;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.validation.validator.StringValidator;

import br.gov.mj.infra.wicket.component.ComponentFactory;
import br.gov.mj.infra.wicket.page.BasePage;
import br.gov.mj.side.entidades.entidade.Pessoa;
import br.gov.mj.side.entidades.entidade.PessoaEntidade;
import br.gov.mj.side.web.service.EntidadeService;
import br.gov.mj.side.web.util.SideSession;
import br.gov.mj.side.web.view.consultaPublica.ConsultaPublicaPesquisaPage;
import br.gov.mj.side.web.view.seguranca.RecuperarSenhaPage;

public class LoginPage extends BasePage {
    private static final long serialVersionUID = 1L;

    private String login;
    private String senha;

    @Inject
    private EntidadeService entidadeService;
    
    @Inject
    private ComponentFactory factory;

    public LoginPage(final PageParameters parameters) {
        super(parameters);

        StatelessForm<LoginPage> form = new StatelessForm<LoginPage>("form") {
            private static final long serialVersionUID = 1L;

            @Override
            protected void onSubmit() {
                boolean result = AuthenticatedWebSession.get().signIn(login, senha);
                if (result) {
                    adicionarEntidadeRepresentanteNaSessao();
                    
                    /* Se o usuário não tiver tentando acessar uma pagina
                     * anteriormente, simplesmente manda-lo para tela Home */                                       
                    Url url = RestartResponseAtInterceptPageException.getOriginalUrl();
                    if (url == null) {
                            setResponsePage(HomePage.class); //manda pra home
                    } else {
                            //envia o usuário pra onde ele tava tentando ir originalmente
                            continueToOriginalDestination();
                    }       
                } else {
                    addMsgError("Usuário e/ou senha inválido(s).");
                }
            }

        };


        FeedbackPanel feedbackPanel = new FeedbackPanel("feedbackPanel");
        feedbackPanel.setOutputMarkupPlaceholderTag(true);
        form.add(feedbackPanel);

        form.setDefaultModel(new CompoundPropertyModel<LoginPage>(this));
        form.add(factory.newTextField("login", "Login", true, null, StringValidator.maximumLength(50)));

        PasswordTextField txSenha = new PasswordTextField("senha");
        txSenha.setLabel(new Model<String>("Senha"));
        txSenha.add(StringValidator.maximumLength(50));
        form.add(txSenha);

        SubmitLink entrarLink = new SubmitLink("entrarLink",form);
        form.add(entrarLink);
        form.setDefaultButton(entrarLink);
        
        BookmarkablePageLink<Void> linkAcesso = new BookmarkablePageLink<Void>("linkRecuperarSenha", RecuperarSenhaPage.class);
        form.add(linkAcesso);

        add(form);

        add(factory.newLabel("labelVersion", getImplementationVersion()));
        
        adicionarMenu();
    }
    
    
    
    private void adicionarMenu() {

        PanelMenu menu = new PanelMenu("panelMenu");
        menu.setVisible(isMenuVisible());
        add(menu);

    }
    
    private class PanelMenu extends WebMarkupContainer {

        private static final long serialVersionUID = 1L;

        public PanelMenu(String id) {
            super(id);

            BookmarkablePageLink<Void> consultaPublica = new BookmarkablePageLink<Void>("pesquisaAberta", ConsultaPublicaPesquisaPage.class);
            add(consultaPublica);
        }
    }
    
    protected boolean isMenuVisible() {
        return true;
    }

    
    private void adicionarEntidadeRepresentanteNaSessao(){
        
        if(getSideSession().getUsuarioLogado()!= null){
            List<PessoaEntidade> entidades = entidadeService.buscarPessoaEntidadesDoUsuario(getSideSession().getUsuarioLogado());           
            if(!entidades.isEmpty()){
                Pessoa pessoa = entidades.get(0).getPessoa();
                if(pessoa.isRepresentante()){
                    addSessionAttribute("entidade",entidades.get(0).getEntidade());
                }
            }
            
            PessoaEntidade pessoaEntidade = entidadeService.buscarPessoaEntidadeDoUsuario(getSideSession().getUsuarioLogado());
            if (pessoaEntidade != null){
                addSessionAttribute("pessoaEntidade", pessoaEntidade);
            }
        }
        
        
        
    }
    
    protected SideSession getSideSession() {
        return SideSession.get();
    }
    
    
}
