package br.gov.mj.side.web.view.beneficiario;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.inject.Inject;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.authroles.authorization.strategies.role.annotations.AuthorizeInstantiation;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.OrderByBorder;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Button;
import org.apache.wicket.markup.html.form.ChoiceRenderer;
import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.IChoiceRenderer;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.markup.repeater.data.DataView;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.validation.validator.StringValidator;

import br.gov.mj.apoio.entidades.Municipio;
import br.gov.mj.apoio.entidades.TipoEntidade;
import br.gov.mj.apoio.entidades.Uf;
import br.gov.mj.infra.wicket.component.ComponentFactory;
import br.gov.mj.infra.wicket.component.InfraAjaxConfirmButton;
import br.gov.mj.infra.wicket.component.InfraAjaxFallbackLink;
import br.gov.mj.infra.wicket.component.InfraDropDownChoice;
import br.gov.mj.infra.wicket.component.pagination.InfraAjaxPagingNavigator;
import br.gov.mj.infra.wicket.model.LambdaModel;
import br.gov.mj.side.entidades.entidade.Entidade;
import br.gov.mj.side.entidades.enums.EnumPersonalidadeJuridica;
import br.gov.mj.side.entidades.enums.EnumStatusEntidade;
import br.gov.mj.side.entidades.enums.EnumTipoPessoa;
import br.gov.mj.side.web.dto.EntidadePesquisaDto;
import br.gov.mj.side.web.service.EntidadeService;
import br.gov.mj.side.web.service.TipoEntidadeService;
import br.gov.mj.side.web.service.UfService;
import br.gov.mj.side.web.util.CnpjUtil;
import br.gov.mj.side.web.util.Constants;
import br.gov.mj.side.web.util.SortableEntidadeDataProvider;
import br.gov.mj.side.web.view.template.TemplatePage;

@AuthorizeInstantiation({ BeneficiarioPesquisaPage.ROLE_MANTER_BENEFICIARIO_INCLUIR, BeneficiarioPesquisaPage.ROLE_MANTER_BENEFICIARIO_ALTERAR, BeneficiarioPesquisaPage.ROLE_MANTER_BENEFICIARIO_CONSULTAR })
public class BeneficiarioPesquisaPage extends TemplatePage {
    private static final long serialVersionUID = 1L;

    public static final String ROLE_MANTER_BENEFICIARIO_INCLUIR = "manter_beneficiario:incluir";
    public static final String ROLE_MANTER_BENEFICIARIO_ALTERAR = "manter_beneficiario:alterar";
    public static final String ROLE_MANTER_BENEFICIARIO_CONSULTAR = "manter_beneficiario:consultar";

    private PanelPrincipal panelPrincipal;
    private PanelGridResultados panelResultados;

    private Uf nomeUf = new Uf();

    private Form<EntidadePesquisaDto> form;
    private EntidadePesquisaDto entidade = new EntidadePesquisaDto();
    private Boolean mostrarResultados = false;
    private SortableEntidadeDataProvider dp;
    private Integer itensPorPagina = Constants.ITEMS_PER_PAGE_PAGINATION;
    private EnumTipoPessoa tipoPessoa = null;

    private DataView<Entidade> dataView;

    @Inject
    private ComponentFactory componentFactory;

    @Inject
    private TipoEntidadeService tipoEntidadeService;

    @Inject
    private UfService ufService;

    @Inject
    private EntidadeService entidadeService;

    public BeneficiarioPesquisaPage(final PageParameters pageParameters) {
        super(pageParameters);
        setTitulo("Pesquisar Beneficiário");

        form = componentFactory.newForm("form", new CompoundPropertyModel<EntidadePesquisaDto>(entidade));
        add(form);

        InfraAjaxFallbackLink<Void> btnNovo = getButtonNovo();
        authorize(btnNovo, RENDER, BeneficiarioPesquisaPage.ROLE_MANTER_BENEFICIARIO_INCLUIR);
        form.add(btnNovo);

        Button btnPesquisar = getButtonPesquisar();
        authorize(btnPesquisar, RENDER, BeneficiarioPesquisaPage.ROLE_MANTER_BENEFICIARIO_CONSULTAR);
        form.add(btnPesquisar);

        panelPrincipal = new PanelPrincipal("panelPrincipal");
        panelResultados = new PanelGridResultados("panelResultados");
        panelResultados.setVisible(false);
        form.add(panelPrincipal);
        form.add(panelResultados);
    }

    private class PanelPrincipal extends WebMarkupContainer {
        public PanelPrincipal(String id) {
            super(id);
            setOutputMarkupId(true);

            add(getTextFieldCnpj()); // entidade.numeroCnpj
            add(getTextFieldNomeEntidade()); // entidade.nomeEntidade
            add(getDropDownTipo()); // entidade.tipoEntidade
            add(getDropDownNaturezaJuridica()); // entidade.personalidadeJuridica
            add(getDropDownStatus()); // entidade.statusEntidade
            add(getTextFieldCpfRepresentante()); // representante.numeroCpf
            add(getTextFieldNomeRepresentante()); // representante.nomeRepresentante
            add(getTextFieldCpfTitular()); // titular.numeroCpf
            add(getTextFieldNomeTitular()); // titular.nomeTitular
            add(getDropDownUf()); // nomeUf
        }
    }

    private class PanelGridResultados extends WebMarkupContainer {
        public PanelGridResultados(String id) {
            super(id);
            setOutputMarkupId(true);
            form.getModelObject().setUsuarioLogado(getUsuarioLogadoDaSessao());
            dp = new SortableEntidadeDataProvider(entidadeService, form.getModelObject());
            add(getDataViewResultado(dp));
            add(getDropItensPorPagina()); // itensPorPagina
            add(new InfraAjaxPagingNavigator("pagination", dataView));

            add(new OrderByBorder<String>("orderByCpf", "numeroCnpj", dp));
            add(new OrderByBorder<String>("orderByNome", "nomeEntidade", dp));
            add(new OrderByBorder<String>("orderByTipo", "tipoEntidade", dp));
            add(new OrderByBorder<String>("orderByEstado", "municipio.uf.siglaUf", dp));
            add(new OrderByBorder<String>("orderBySituacao", "statusEntidade", dp));

        }
    }

    // COMPONENTES

    private TextField<String> getTextFieldCnpj() {

        TextField<String> field = componentFactory.newTextField("entidade.numeroCnpj", "CNPJ", false, null);
        field.add(StringValidator.maximumLength(18));
        return field;
    }

    private TextField<String> getTextFieldNomeEntidade() {
        TextField<String> field = componentFactory.newTextField("entidade.nomeEntidade", "Nome (Entidade)", false, null);
        field.add(StringValidator.maximumLength(200));
        return field;
    }

    public DropDownChoice<TipoEntidade> getDropDownTipo() {
        List<TipoEntidade> lista = new ArrayList<TipoEntidade>();
        lista = tipoEntidadeService.buscarTodos();

        InfraDropDownChoice<TipoEntidade> drop = componentFactory.newDropDownChoice("entidade.tipoEntidade", "Tipo", false, "id", "descricaoTipoEntidade", null, lista, null);
        drop.setNullValid(true);
        drop.setOutputMarkupId(true);

        return drop;
    }

    public DropDownChoice<EnumPersonalidadeJuridica> getDropDownNaturezaJuridica() {
        List<EnumPersonalidadeJuridica> listaTemp = Arrays.asList(EnumPersonalidadeJuridica.values());
        List<EnumPersonalidadeJuridica> lista = new ArrayList<EnumPersonalidadeJuridica>();
        for (EnumPersonalidadeJuridica temp : listaTemp) {
            if (!"T".equalsIgnoreCase(temp.getValor())) {
                lista.add(temp);
            }
        }

        InfraDropDownChoice<EnumPersonalidadeJuridica> drop = componentFactory.newDropDownChoice("entidade.personalidadeJuridica", "Natureza Jurídica", false, "valor", "descricao", null, lista, null);
        drop.setChoiceRenderer(criarRendererNaturezaJuridica());
        drop.setNullValid(true);
        drop.setOutputMarkupId(true);

        return drop;
    }

    public DropDownChoice<EnumStatusEntidade> getDropDownStatus() {
        List<EnumStatusEntidade> listaTemp = Arrays.asList(EnumStatusEntidade.values());
        /*
         * List<EnumPersonalidadeJuridica> lista = new
         * ArrayList<EnumPersonalidadeJuridica>(); for
         * (EnumPersonalidadeJuridica temp : listaTemp) { if
         * (!"T".equalsIgnoreCase(temp.getValor())) { lista.add(temp); } }
         */

        InfraDropDownChoice<EnumStatusEntidade> drop = componentFactory.newDropDownChoice("entidade.statusEntidade", "Situação", false, "valor", "descricao", null, listaTemp, null);
        // drop.setChoiceRenderer(criarRendererNaturezaJuridica());
        drop.setNullValid(true);
        drop.setOutputMarkupId(true);

        return drop;
    }

    private TextField<String> getTextFieldCpfRepresentante() {
        TextField<String> field = componentFactory.newTextField("representante.numeroCpf", "CPF (Representante)", false, null);
        field.add(StringValidator.maximumLength(14));
        return field;
    }

    private TextField<String> getTextFieldNomeRepresentante() {

        TextField<String> field = componentFactory.newTextField("representante.nomePessoa", "Nome (Representante)", false, null);
        field.add(StringValidator.maximumLength(200));
        actionTextFieldRepresentante(field);
        return field;
    }

    private TextField<String> getTextFieldCpfTitular() {
        TextField<String> field = componentFactory.newTextField("titular.numeroCpf", "CPF (Titular)", false, null);
        field.add(StringValidator.maximumLength(14));
        return field;
    }

    private TextField<String> getTextFieldNomeTitular() {
        TextField<String> field = componentFactory.newTextField("titular.nomePessoa", "Nome (Titular)", false, null);
        field.add(StringValidator.maximumLength(200));
        actionTextFieldTitular(field);
        return field;
    }

    public DropDownChoice<Uf> getDropDownUf() {

        List<Uf> lista = ufService.buscarTodos();

        InfraDropDownChoice<Uf> drop = componentFactory.newDropDownChoice("nomeUf", "Estado", false, "id", "nomeUf", new PropertyModel(this, "nomeUf"), lista, null);
        drop.setChoiceRenderer(new ChoiceRenderer<Uf>("nomeSigla", "id"));
        drop.setNullValid(true);
        drop.setOutputMarkupId(true);

        return drop;
    }

    private DataView<Entidade> getDataViewResultado(SortableEntidadeDataProvider dp) {
        dataView = new DataView<Entidade>("entidades", dp) {
            private static final long serialVersionUID = 1L;

            @Override
            protected void populateItem(Item<Entidade> item) {

                item.add(new Label("numeroCnpj", CnpjUtil.imprimeCNPJ(item.getModelObject().getNumeroCnpj())));
                item.add(new Label("nomeEntidade"));
                item.add(new Label("tipoEntidade", item.getModelObject().getTipoEntidade().getDescricaoTipoEntidade()));
                item.add(new Label("estado", item.getModelObject().getMunicipio().getUf().getNomeSigla()));
                item.add(new Label("statusEntidade.descricao"));

                Button btnAlterar = componentFactory.newButton("btnAlterar", () -> alterar(item));
                authorize(btnAlterar, RENDER, BeneficiarioPesquisaPage.ROLE_MANTER_BENEFICIARIO_ALTERAR);
                item.add(btnAlterar);

                Button btnVisualizar = getButtonVisualizar(item);
                authorize(btnVisualizar, RENDER, BeneficiarioPesquisaPage.ROLE_MANTER_BENEFICIARIO_CONSULTAR);
                item.add(btnVisualizar);

            }
        };
        dataView.setItemsPerPage(Constants.ITEMS_PER_PAGE_PAGINATION);
        return dataView;
    }

    public InfraAjaxConfirmButton getButtonExcluir(Item<Entidade> item) {
        return componentFactory.newAJaxConfirmButton("btnExcluir", "MSG001", null, (target, formz) -> excluirEntidade(target, item));
    }

    public Button getButtonVisualizar(Item<Entidade> item) {
        return componentFactory.newButton("btnVisualizar", () -> visualizar(item));
    }

    private DropDownChoice<Integer> getDropItensPorPagina() {
        DropDownChoice<Integer> dropDownChoice = new DropDownChoice<Integer>("itensPorPagina", new LambdaModel<Integer>(this::getItensPorPagina, this::setItensPorPagina), Arrays.asList(5, 10, 15, 20, 25, 30));
        dropDownChoice.add(new AjaxFormComponentUpdatingBehavior("onchange") {
            private static final long serialVersionUID = 1L;

            @Override
            protected void onUpdate(AjaxRequestTarget target) {
                dataView.setItemsPerPage(getItensPorPagina());
                target.add(panelResultados);
            };
        });
        return dropDownChoice;
    }

    public InfraAjaxFallbackLink<Void> getButtonNovo() {
        return componentFactory.newAjaxFallbackLink("btnNovo", (target) -> adicionarNovo());
    }

    private Button getButtonPesquisar() {
        return componentFactory.newButton("btnPesquisar", () -> pesquisar());
    }

    // AÇÕES

    public void excluirEntidade(AjaxRequestTarget target, Item<Entidade> item) {
        Entidade entidade = item.getModelObject();
        entidadeService.excluir(entidade.getId());
        addMsgInfo("MN032");
        target.add(panelResultados);
    }

    private void visualizar(Item<Entidade> item) {
        setResponsePage(new BeneficiarioPage(new PageParameters(), this, item.getModelObject(), true));
    }

    private IChoiceRenderer criarRendererNaturezaJuridica() {
        return new IChoiceRenderer<EnumPersonalidadeJuridica>() {
            private static final long serialVersionUID = 1L;

            @Override
            public Object getDisplayValue(EnumPersonalidadeJuridica object) {
                if ("I".equalsIgnoreCase(object.getValor())) {
                    return "Privada Sem Fins Lucrativos";
                } else {
                    return object.getDescricao();
                }
            }

            @Override
            public String getIdValue(EnumPersonalidadeJuridica object, int index) {
                return object.toString();
            }
        };
    }

    private void alterar(Item<Entidade> item) {
        if (!getSideSession().hasRole(ROLE_MANTER_BENEFICIARIO_ALTERAR)) {
            throw new SecurityException();
        }
        setResponsePage(new BeneficiarioPage(new PageParameters(), this, item.getModelObject(), false));
    }

    private void adicionarNovo() {
        if (!getSideSession().hasRole(ROLE_MANTER_BENEFICIARIO_INCLUIR)) {
            throw new SecurityException();
        }
        setResponsePage(new BeneficiarioPage(null, this, new Entidade(), false));
    }

    private void pesquisar() {
        if (!getSideSession().hasRole(ROLE_MANTER_BENEFICIARIO_CONSULTAR)) {
            throw new SecurityException();
        }

        EntidadePesquisaDto dto = new EntidadePesquisaDto();
        dto = form.getModelObject();

        if (dto.getEntidade() != null) {
            dto.getEntidade().setMunicipio(new Municipio());
        }

        if (dto.getEntidade() != null && dto.getEntidade().getNumeroCnpj() != null && !"".equalsIgnoreCase(dto.getEntidade().getNumeroCnpj())) {
            dto.getEntidade().setNumeroCnpj(limparCampos(dto.getEntidade().getNumeroCnpj()));
        }

        if (nomeUf != null && nomeUf.getId() != null) {
            Municipio municipio = new Municipio();
            municipio.setUf(nomeUf);
            if (dto.getEntidade() == null) {
                Entidade ent = new Entidade();
                ent.setMunicipio(municipio);
                dto.setEntidade(ent);
            } else {
                dto.getEntidade().setMunicipio(municipio);
            }
        }
        // List<Entidade> lista = entidadeService.buscarSemPaginacao(dto);
        mostrarResultados = true;
        panelResultados.setVisible(true);
    }

    // Actions

    private void actionTextFieldRepresentante(TextField<String> field) {
        AjaxFormComponentUpdatingBehavior onChangeAjaxBehavior = new AjaxFormComponentUpdatingBehavior("onchange") {
            @Override
            protected void onUpdate(AjaxRequestTarget target) {

                tipoPessoa = EnumTipoPessoa.REPRESENTANTE_ENTIDADE;
            }
        };
        field.add(onChangeAjaxBehavior);
    }

    private void actionTextFieldTitular(TextField<String> field) {
        AjaxFormComponentUpdatingBehavior onChangeAjaxBehavior = new AjaxFormComponentUpdatingBehavior("onchange") {
            @Override
            protected void onUpdate(AjaxRequestTarget target) {

                tipoPessoa = EnumTipoPessoa.TITULAR;
            }
        };
        field.add(onChangeAjaxBehavior);
    }

    private String limparCampos(String valor) {
        String value = valor;
        value = value.replace(".", "");
        value = value.replace("/", "");
        value = value.replace("-", "");
        value = value.replace("(", "");
        value = value.replace(")", "");
        return value;
    }

    public Integer getItensPorPagina() {
        return itensPorPagina;
    }

    public void setItensPorPagina(Integer itensPorPagina) {
        this.itensPorPagina = itensPorPagina;
    }
}
