package br.gov.mj.side.web.view.dashboard;

import javax.inject.Inject;

import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.hibernate.SQLQuery.ReturnProperty;

import br.gov.mj.infra.wicket.component.ComponentFactory;
import br.gov.mj.infra.wicket.component.InfraAjaxFallbackLink;
import br.gov.mj.side.entidades.entidade.Entidade;
import br.gov.mj.side.entidades.entidade.LocalEntregaEntidade;
import br.gov.mj.side.entidades.entidade.PessoaEntidade;
import br.gov.mj.side.entidades.enums.EnumTipoPessoa;
import br.gov.mj.side.web.service.EntidadeService;
import br.gov.mj.side.web.view.HomePage;
import br.gov.mj.side.web.view.dashboard.paineis.PanelDadosEntidade;
import br.gov.mj.side.web.view.dashboard.paineis.PanelDadosPessoa;
import br.gov.mj.side.web.view.programa.inscricao.locaisEntrega.EntidadeNavPanel;
import br.gov.mj.side.web.view.template.TemplatePage;


public class InformacoesEntidadePage extends TemplatePage {
    
	private static final long serialVersionUID = 1L;
	//###################################################################################
	//constantes

	//###################################################################################
	//variaveis
	private Entidade						entidade;
	
	//elementos do Wicket
	private	Form<Entidade>					form;
	private PanelDadosEntidade				panelDadosEntidade;
	private PanelDadosPessoa				panelDadosPessoa;

	//###################################################################################
	//injeçãod e dependencia
	@Inject private	ComponentFactory		componentFactory;
	@Inject private EntidadeService			entidadeService;
	
	
	//###################################################################################
	//conscructs, inits & destroiers
	public InformacoesEntidadePage(PageParameters pageParameters) {
		super(pageParameters);
		setTitulo("Informações Entidade");
		PessoaEntidade pessoaEntidade=(PessoaEntidade)getSessionAttribute("pessoaEntidade");
		
		if(pessoaEntidade!=null){
			entidade=pessoaEntidade.getEntidade();
			entidade.setLocaisEntregaEntidade(entidadeService.buscarLocaisEntrega(entidade, null));
			entidade.setPessoas(entidadeService.buscarTitularEntidade(entidade));
		}
		form	=	componentFactory.newForm("form",  new CompoundPropertyModel<Entidade>(entidade));
		initPage();
	}
	
	public void initPage(){
		
	    form.add(new EntidadeNavPanel("navPanel",this));
		form.add(newPanelDadosEntidade());
		form.add(newPanelDadosPessoa());
		form.add(newHomePageLink());
		form.add(getButtonVoltar());
		add(form);
	}

	//###################################################################################
	//classes privadas

	//###################################################################################
	//criação e configuração de componentes
	private PanelDadosEntidade newPanelDadosEntidade(){
		panelDadosEntidade=new PanelDadosEntidade("panelDadosEntidade",form,entidade);
		panelDadosEntidade.enableReadOnly();
		
		return panelDadosEntidade;
	}
	
	private PanelDadosPessoa newPanelDadosPessoa(){
		panelDadosPessoa =new PanelDadosPessoa("panelDadosTitular", form,  entidade.getPessoas(),"Titulares",EnumTipoPessoa.TITULAR);
		panelDadosPessoa.enableHideMainPanelWhenEmpty();
		panelDadosPessoa.enableReadOnly();
		
		return panelDadosPessoa;
	}
	/**
	 * retorn o link para o dashboard
	 * @return
	 */
	public Link newHomePageLink(){
		Link link=new Link("homePage") {
           @Override
           public void onClick() {
               setResponsePage(HomePage.class);
           }
       };
		
		return link;
	}
	/**
	 * recupera o botão com coportamento de voltar para a home
	 * @return
	 */
	public InfraAjaxFallbackLink<Void> getButtonVoltar(){
		InfraAjaxFallbackLink<Void> button=componentFactory.newAjaxFallbackLink("btnVoltar", (target)->  setResponsePage(HomePage.class));
		
		return button;
	}

}
