package br.gov.mj.side.web.view.fornecedor;

import javax.inject.Inject;

import org.apache.wicket.Page;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Button;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;

import br.gov.mj.infra.wicket.component.ComponentFactory;
import br.gov.mj.side.entidades.entidade.Entidade;
import br.gov.mj.side.entidades.entidade.PessoaEntidade;
import br.gov.mj.side.entidades.enums.EnumTipoPessoa;
import br.gov.mj.side.web.service.EntidadeService;
import br.gov.mj.side.web.view.HomePage;
import br.gov.mj.side.web.view.dashboard.paineis.PanelDadosPessoa;
import br.gov.mj.side.web.view.fornecedor.paineis.DadosFornecedorPanel;
import br.gov.mj.side.web.view.fornecedor.paineis.PanelDadosPessoaFornecedor;
import br.gov.mj.side.web.view.template.TemplatePage;

/**
 * Classe destinada a pagina de cadastro de fornecedores
 * 
 * @author diego.mota & ronald.calazans
 *
 */
public class FornecedorPage extends TemplatePage {
    private static final long serialVersionUID = 1L;
	//###################################################################################
	//constantes

	//###################################################################################
	//variaveis
    private Entidade						fornecedor;
    
    private PanelBotoes 					panelBotoes;	
    private Form<Entidade>					form;
    private Button							buttonSalvar;
    private Page 							backPage;
    private Label 							lblBread;
    private DadosFornecedorPanel			dadosFornecedorPanel;
    private PanelDadosPessoaFornecedor		panelDadosPessoa;

	//###################################################################################
	//injeçãod e dependencia
    @Inject private ComponentFactory	componentFactory;
    @Inject	private EntidadeService		entidadeService;

	//###################################################################################
	//conscructs, inits & destroiers
    
    public FornecedorPage(final PageParameters pageParameters){
    	super(pageParameters);
    	initComponents();
    }
    
    public FornecedorPage(final PageParameters pageParameters, Page backPage) {
        super(pageParameters);
        this.backPage = backPage;
        
        initComponents();
    }

    private void initComponents() {
//    	criarBreadcrump();
        setTitulo("Cadastrar Fornecedor");
        
    	PessoaEntidade pessoaEntidade=(PessoaEntidade)getSessionAttribute("pessoaEntidade");
		
		if(pessoaEntidade!=null){
			fornecedor=pessoaEntidade.getEntidade();
			fornecedor.setLocaisEntregaEntidade(entidadeService.buscarLocaisEntrega(fornecedor, null));
			fornecedor.setPessoas(entidadeService.buscarTitularEntidade(fornecedor));
		}else{
			fornecedor=new Entidade();
		}

		
        form =new Form<Entidade>("form",new CompoundPropertyModel<Entidade>(fornecedor));
       
        form.add(newDadosFornecedorPanel());
        form.add(newPanelDadosPessoa("panelRepresentantes","Titulares"));
        form.add(newPanelDadosPessoa("panelPrepostos","Prepostos")); //ESSE APAINEL ESTA CONFIGURADO PARA REPRESENTANTE POIS O TIPO 'PREPOSTO' AINDA NÃO EXISTE
        form.add(panelBotoes = new PanelBotoes("panelBotoes"));
        
        form.add(lblBread = new Label("lblBread","Cadastrar Fornecedor"));
        add(form);
    }
	//###################################################################################
	//classes privadas

    //paineis
    private class PanelBotoes extends WebMarkupContainer {
        private static final long serialVersionUID = 1L;

        public PanelBotoes(String id) {
            super(id);
            setOutputMarkupId(true);
            add(newButtonSalvar()); // btnSalvar
            add(newButtonVoltar()); // btnVoltar
        }
    }
    
	//###################################################################################
	//criação e configuração de componentes
    private void criarBreadcrump() {
        Link link = new Link("homePage") {
            @Override
            public void onClick() {
                setResponsePage(HomePage.class);
            }
        };
        form.add(link);

        form.add(new Link("fornecedorPesquisaPage") {
            @Override
            public void onClick() {
                setResponsePage(backPage);
            }
        });
    }
    
    private DadosFornecedorPanel newDadosFornecedorPanel(){
    	dadosFornecedorPanel=new DadosFornecedorPanel("panelDadosFornecedor", form, fornecedor);
    	return dadosFornecedorPanel;
    }
    
    private PanelDadosPessoaFornecedor newPanelDadosPessoa(String id, String tituloPanel){
    	panelDadosPessoa=new PanelDadosPessoaFornecedor(id, form, form.getModelObject().getPessoas(),tituloPanel,EnumTipoPessoa.REPRESENTANTE_FORNECEDOR);
    	
    	return panelDadosPessoa;
    }
    
    private Button newButtonVoltar() {
        Button button = componentFactory.newButton("btnVoltar", () -> actionVoltar());
        button.setDefaultFormProcessing(false);
        return button;
    }
    
    private Button newButtonSalvar() {
        buttonSalvar = componentFactory.newButton("btnSalvar", () -> actionSalvar());
        buttonSalvar.setOutputMarkupId(true);        
        return buttonSalvar;
    }
   
	//###################################################################################
	//metodos privados
    
    
	//###################################################################################
	//metodos publicos
    
    private void actionVoltar() {
        setResponsePage(backPage);
    }
    
    private void actionSalvar()
    {
        
    }    
}
