package br.gov.mj.side.web.view.fornecedor;

import javax.inject.Inject;

import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.form.Button;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.validation.validator.StringValidator;

import br.gov.mj.apoio.entidades.Uf;
import br.gov.mj.infra.wicket.component.ComponentFactory;
import br.gov.mj.infra.wicket.component.InfraAjaxFallbackLink;
import br.gov.mj.infra.wicket.component.InfraDropDownChoice;
import br.gov.mj.side.web.service.UfService;
import br.gov.mj.side.web.view.HomePage;
import br.gov.mj.side.web.view.template.TemplatePage;

/**
 * Classe destinada a pagina de pesquisa de fornecedores
 * 
 * @author diego.mota
 *
 */
public class FornecedorPesquisaPage extends TemplatePage {
    private static final long serialVersionUID = 1L;
    
    //##################################################################################
    //variaveis
    private String 							temporario;
    private Uf 								estadoSelecionado;
    
    //compoentes do wicket
    private PanelPrincipal 					panelPrincipal;
    private Form<FornecedorPesquisaPage> 	form;

    //injeção de dependencias
    
    @Inject private UfService 				ufService;
    @Inject private ComponentFactory 		componentFactory;
    
  
    //##################################################################################
    //construtores,intis * destroyers
    public FornecedorPesquisaPage(final PageParameters pageParameters) {
        super(pageParameters);
        
        initComponents();
        criarBreadcrump();
    }

    private void initComponents() {
        form =new Form<FornecedorPesquisaPage>("form",new CompoundPropertyModel<FornecedorPesquisaPage>(this));
        add(form);
        
        form.add(panelPrincipal = new PanelPrincipal("panelPrincipal"));
        
        form.add(newButtonNovo()); //btnNovo
        form.add(newButtonPesquisar()); //btnPesquisar
    }

    //##################################################################################
    //componentes wicket    
    private void criarBreadcrump() {
        Link link = new Link("homePage") {
            @Override
            public void onClick() {
                setResponsePage(HomePage.class);
            }
        };
        form.add(link);        
    }
    
    /*
     * OS PAINEIS SERÃO IMPLEMENTADOS ABAIXO
     */
    
    private class PanelPrincipal extends WebMarkupContainer {
        public PanelPrincipal(String id) {
            super(id);
            setOutputMarkupId(true);

            add(newTextFieldCnpj());
            add(newTextFieldRazãoSocial());
            add(newDropDownUf());
            add(newTextFieldContrato());
            add(newTextFieldPrograma());
            add(newTextFieldTipoDeBem());
        }
    }
    
    
    
    /*
     * OS COMPONENTES SERÃO IMPLEMENTADOS ABAIXO
     */
    
    private TextField<String> newTextFieldCnpj() {

        TextField<String> field = componentFactory.newTextField("cnpj", "CNPJ", false, new PropertyModel<String>(this, "temporario"));
        field.add(StringValidator.maximumLength(18));
        return field;
    }
    
    private TextField<String> newTextFieldRazãoSocial() {

        TextField<String> field = componentFactory.newTextField("razaoSocial", "Razão Social", false, new PropertyModel<String>(this, "temporario"));
        field.add(StringValidator.maximumLength(18));
        return field;
    }
    
    private InfraDropDownChoice<Uf> newDropDownUf() {
        InfraDropDownChoice<Uf> dropDownUf = componentFactory.newDropDownChoice("dropEstado", "Estado", false, "id", "nomeSigla", new PropertyModel<Uf>(this,"estadoSelecionado"), ufService.buscarTodos(), null);
        dropDownUf.setNullValid(true);
        return dropDownUf;
    }
    
    private TextField<String> newTextFieldContrato() {

        TextField<String> field = componentFactory.newTextField("contrato", "Contrato", false, new PropertyModel<String>(this, "temporario"));
        field.add(StringValidator.maximumLength(18));
        return field;
    }
    
    private TextField<String> newTextFieldPrograma() {

        TextField<String> field = componentFactory.newTextField("programa", "Programa", false, new PropertyModel<String>(this, "temporario"));
        field.add(StringValidator.maximumLength(18));
        return field;
    }
    
    private TextField<String> newTextFieldTipoDeBem() {

        TextField<String> field = componentFactory.newTextField("tipoBem", "Tipo de Bem", false, new PropertyModel<String>(this, "temporario"));
        field.add(StringValidator.maximumLength(18));
        return field;
    }
    
    public InfraAjaxFallbackLink<Void> newButtonNovo() {
        return componentFactory.newAjaxFallbackLink("btnNovo", (target) -> adicionarNovo());
    }
    
    private Button newButtonPesquisar() {
        return componentFactory.newButton("btnPesquisar", () -> pesquisar());
    }
    
    /*
     * AS AÇÕES SERÃO IMPLEMENTADAS ABAIXO
     */
    
    private void pesquisar()
    {
        
    }
    
    private void adicionarNovo()
    {
        setResponsePage(new FornecedorPage(getPageParameters(), this));
    }

    
}
