package br.gov.mj.side.web.view.planejarLicitacao;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import javax.inject.Inject;

import org.apache.wicket.Page;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.ajax.markup.html.form.AjaxSubmitLink;
import org.apache.wicket.authroles.authorization.strategies.role.annotations.AuthorizeInstantiation;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Button;
import org.apache.wicket.markup.html.form.ChoiceRenderer;
import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;

import br.gov.mj.infra.wicket.component.ComponentFactory;
import br.gov.mj.infra.wicket.model.LambdaModel;
import br.gov.mj.side.entidades.Bem;
import br.gov.mj.side.entidades.enums.EnumStatusPrograma;
import br.gov.mj.side.entidades.enums.EnumTipoAgrupamentoLicitacao;
import br.gov.mj.side.entidades.programa.Programa;
import br.gov.mj.side.entidades.programa.inscricao.InscricaoPrograma;
import br.gov.mj.side.entidades.programa.inscricao.analise.ListaAvaliacaoPublicado;
import br.gov.mj.side.entidades.programa.licitacao.AgrupamentoLicitacao;
import br.gov.mj.side.entidades.programa.licitacao.BemUf;
import br.gov.mj.side.entidades.programa.licitacao.LicitacaoPrograma;
import br.gov.mj.side.entidades.programa.licitacao.SelecaoItem;
import br.gov.mj.side.web.dto.BemUfDto;
import br.gov.mj.side.web.dto.ItemBemDto;
import br.gov.mj.side.web.dto.PesquisaLicitacaoDto;
import br.gov.mj.side.web.service.InscricaoProgramaService;
import br.gov.mj.side.web.service.LicitacaoProgramaService;
import br.gov.mj.side.web.service.ProgramaService;
import br.gov.mj.side.web.view.HomePage;
import br.gov.mj.side.web.view.components.AjaxDialogButton;
import br.gov.mj.side.web.view.template.TemplatePage;
import de.agilecoders.wicket.core.markup.html.bootstrap.button.Buttons;
import de.agilecoders.wicket.core.markup.html.bootstrap.dialog.Modal;
import de.agilecoders.wicket.core.markup.html.bootstrap.dialog.Modal.Backdrop;
import de.agilecoders.wicket.core.markup.html.bootstrap.dialog.TextContentModal;

@AuthorizeInstantiation({ 
    PlanejamentoLicitacaoPage.ROLE_MANTER_PLANEJAR_LICITACAO_VISUALIZAR,
    PlanejamentoLicitacaoPage.ROLE_MANTER_PLANEJAR_LICITACAO_INCLUIR,
    PlanejamentoLicitacaoPage.ROLE_MANTER_PLANEJAR_LICITACAO_ALTERAR,
    PlanejamentoLicitacaoPage.ROLE_MANTER_PLANEJAR_LICITACAO_EXCLUIR
})

public class PlanejamentoLicitacaoPage extends TemplatePage {
    private static final long serialVersionUID = 1L;
    
  //constantes de permição de acesso
    public static final String ROLE_MANTER_PLANEJAR_LICITACAO_VISUALIZAR = "manter_planejar_licitacao:visualizar";
    public static final String ROLE_MANTER_PLANEJAR_LICITACAO_INCLUIR = "manter_planejar_licitacao:incluir";
    public static final String ROLE_MANTER_PLANEJAR_LICITACAO_ALTERAR = "manter_planejar_licitacao:alterar";
    public static final String ROLE_MANTER_PLANEJAR_LICITACAO_EXCLUIR = "manter_planejar_licitacao:excluir";
    
    private PanelInformacoesBasicasLicitacao panelInformacoesBasicas;
    private PanelItensLicitacao panelItensLicitacao;
    private PanelBotoes panelBotoes;
    
    private Form<LicitacaoPrograma> form;
    private Button buttonSalvar;
    private Page backPage;
    private Label lblBread;
    private AjaxSubmitLink buttonCancelarEdicao;
    private Modal<String> modalLimparFormulario;
    private String msgConfirm;

    private List<Programa> listaProgramas = new ArrayList<Programa>();
    private List<InscricaoPrograma> listaInscricoesProgramaSelecionado = new ArrayList<InscricaoPrograma>();
    private List<BemUfDto> listaLocaisEntrega = new ArrayList<BemUfDto>();
    private List<BemUfDto> listaComDadosOriginaisBemUfDto = new ArrayList<BemUfDto>();
    
    private boolean readOnly;
    private boolean mostrarBotaoTrocarLicitacao;
    
    private LicitacaoPrograma licitacaoPrograma = new LicitacaoPrograma();

    @Inject
    private ComponentFactory componentFactory;
    
    @Inject
    private InscricaoProgramaService inscricaoService;
    
    @Inject
    private ProgramaService programaService;
    
    @Inject
    private LicitacaoProgramaService licitacaoService;
    
    public PlanejamentoLicitacaoPage(final PageParameters pageParameters, Page backPage, LicitacaoPrograma licitacaoPrograma,boolean readOnly) {
        super(pageParameters);
        this.backPage = backPage;
        this.readOnly=readOnly;
        this.licitacaoPrograma = licitacaoPrograma;
                
        listaDeProgramasElegiveis();
        initComponents();
        
        if(licitacaoPrograma != null || licitacaoPrograma.getId() != null){
            preencheAsListasNoModoEditar();
        }
        
        form.getModelObject().setPrograma(licitacaoPrograma.getPrograma());
        
        criarBreadcrump();
        setTitulo(gerarTituloPagina());
    }
        
    private void initComponents() {
        form =new Form<LicitacaoPrograma>("form",new CompoundPropertyModel<LicitacaoPrograma>(new LicitacaoPrograma()));
        add(form);
        
        form.setModelObject(licitacaoPrograma);
        
        form.add(newDropDownPrograma());
        
        form.add(getButtonCancelarEdicao());
        modalLimparFormulario = newModalLimparFormulario("modalLimparFormulario");
        modalLimparFormulario.show(false);

        form.add(modalLimparFormulario);
        
        form.add(panelInformacoesBasicas = new PanelInformacoesBasicasLicitacao("panelInformacoesBasicas",form.getModelObject(),this,readOnly));
        form.add(panelItensLicitacao = new PanelItensLicitacao("panelItensLicitacao",listaLocaisEntrega,licitacaoPrograma, readOnly));
        form.add(panelBotoes = new PanelBotoes("panelBotoes",readOnly));
        
        form.add(lblBread = new Label("lblBread","Planejar Licitação"));
    }
    
    private void criarBreadcrump() {
        Link link = new Link("homePage") {
            private static final long serialVersionUID = 1L;

            @Override
            public void onClick() {
                setResponsePage(HomePage.class);
            }
        };
        form.add(link);

        form.add(new Link("licitacaoPesquisaPage") {
            private static final long serialVersionUID = 1L;

            @Override
            public void onClick() {
                setResponsePage(backPage);
            }
        });
    }

    

    /*
     * OS PAINEIS SÃO ADICIONADOS ABAIXO
     */
    
    private class PanelBotoes extends WebMarkupContainer {
        private static final long serialVersionUID = 1L;

        public PanelBotoes(String id,Boolean readOnly) {
            super(id);
            
            setOutputMarkupId(true);
            add(newButtonSalvar()); // btnSalvar
            add(newButtonVoltar()); // btnVoltar
        }
    }

    /*
     * OS COMPONENTES DOS PAINEIS ESTÃO ABAIXO
     */
    
    public DropDownChoice<Programa> newDropDownPrograma() {
        DropDownChoice<Programa> dropPrograma = new DropDownChoice<Programa>("dropPrograma", new PropertyModel<Programa>(form.getModelObject(), "programa"), listaProgramas, new ChoiceRenderer<Programa>("codigoIdentificadorProgramaPublicadoENomePrograma"));
        dropPrograma.setOutputMarkupId(true);
        dropPrograma.setEnabled(dropProgramaEnabled());
        actionDropDownPrograma(dropPrograma);
        return dropPrograma;
    }  
    
    private AjaxSubmitLink getButtonCancelarEdicao() {
        AjaxSubmitLink buttonCancelarEdicao = new AjaxSubmitLink("btnCancelarEdicao") {
            private static final long serialVersionUID = 1L;

            @Override
            public void onSubmit(AjaxRequestTarget target, Form form) {
                cancelarEdicao(target);
            }
        };
        buttonCancelarEdicao.setVisible(mostrarBotaoTrocarLicitacao());
        buttonCancelarEdicao.setDefaultFormProcessing(false);
        buttonCancelarEdicao.setOutputMarkupId(true);
        return buttonCancelarEdicao;
    }
    
    private void cancelarEdicao(AjaxRequestTarget target){
        msgConfirm = "Esta ação irá desconsiderar todas as alterações realizadas, deseja continuar?.";
        modalLimparFormulario.show(true);
        target.add(modalLimparFormulario);
    }
    
    private Modal<String> newModalLimparFormulario(String id) {
        Modal<String> modal = new TextContentModal(id, new LambdaModel<String>(this::getMsgConfirm, this::setMsgConfirm));
        modal.setBackdrop(Backdrop.STATIC);
        modal.addButton(newButtonLimparFormulario(modal));
        modal.addButton(newButtonFecharModal(modal));
        return modal;
    }
    
    private AjaxDialogButton newButtonLimparFormulario(Modal<String> modal) {
        return new AjaxDialogButton(Model.of("OK"), Buttons.Type.Primary) {
            private static final long serialVersionUID = 1L;

            @Override
            public void onClick(AjaxRequestTarget target) {
                setResponsePage(new PlanejamentoLicitacaoPage(getPageParameters(),backPage,new LicitacaoPrograma(),false));
            }
        };
    }
    
    private AjaxDialogButton newButtonFecharModal(Modal<String> modal) {
        return new AjaxDialogButton(Model.of("Cancelar"), Buttons.Type.Primary) {
            private static final long serialVersionUID = 1L;

            @Override
            public void onClick(AjaxRequestTarget target) {
                modal.show(false);
                modal.close(target);
            }
        };
    }
    
    private Button newButtonVoltar() {
        Button button = componentFactory.newButton("btnVoltar", () -> actionVoltar());
        button.setDefaultFormProcessing(false);
        return button;
    }
    
    private Button newButtonSalvar() {
        buttonSalvar = componentFactory.newButton("btnSalvar", () -> actionSalvar());
        buttonSalvar.setOutputMarkupId(true);   
        buttonSalvar.setEnabled(botoesEnabled());
        authorize(buttonSalvar, RENDER, ROLE_MANTER_PLANEJAR_LICITACAO_INCLUIR,ROLE_MANTER_PLANEJAR_LICITACAO_ALTERAR);
        return buttonSalvar;
    }
    
    
    /*
     * O PROVIDER SERÁ IMPLEMENTADO ABAIXO
     */
    
    

    /*
     * AS AÇÕES SERÃO IMPLEMENTADAS ABAIXO
     */
    
    private void listaDeProgramasElegiveis(){
        List<Programa> programa = new ArrayList<Programa>();
        programa = programaService.buscar(new Programa());
        
        for(Programa prog:programa){
            List<ListaAvaliacaoPublicado> listaAvaliacao = programaService.buscarListaAvaliacaoPublicado(prog);
            if(listaAvaliacao.size()>1 && prog.getStatusPrograma() == EnumStatusPrograma.ABERTO_PLANEJAMENTO_LICITACAO){
                listaProgramas.add(prog);
            }
        }
    }
    
    public void actionDropDownPrograma(DropDownChoice dropPrograma) {
        dropPrograma.add(new AjaxFormComponentUpdatingBehavior("onchange") {
            private static final long serialVersionUID = 1L;

            @Override
            protected void onUpdate(AjaxRequestTarget target) {
                
                PesquisaLicitacaoDto pesquisa = new PesquisaLicitacaoDto();
                pesquisa.setPrograma(form.getModelObject().getPrograma());
                List<LicitacaoPrograma> lista = new ArrayList<LicitacaoPrograma>();
                
                if(form.getModelObject().getPrograma() == null){
                    lista = new ArrayList<LicitacaoPrograma>();
                    setResponsePage(new PlanejamentoLicitacaoPage(getPageParameters(), backPage, new LicitacaoPrograma(), false));
                    return;
                }else{
                    lista = licitacaoService.buscarSemPaginacao(pesquisa);
                }
                
                if(lista.size()>0){
                    setResponsePage(new PlanejamentoLicitacaoPage(getPageParameters(), backPage, lista.get(0), false));
                }else{
                    
                }
                
                //Caso não exista ainda uma licitação salva será buscado os endereços para atualizar na mesma tela.
                atualizarPainelLocaisDeEntrega(target);
             }
        });
    }
    
    private void preencheAsListasNoModoEditar(){
        if(form.getModelObject().getPrograma() == null){
            listaLocaisEntrega = new ArrayList<BemUfDto>();
        }else{
            listaLocaisEntrega = inscricaoService.buscarListaBemUfDto(form.getModelObject().getPrograma());
        }
        copiarConteudoDaListaDeLocaisDeEntrega();
        
        //A lista original sempre será usada quando um item ou grupo for excluido, baseado nela vou voltar a primeira tabela aos valores originais.
        panelItensLicitacao.setListaComDadosOriginaisBemUfDto(listaComDadosOriginaisBemUfDto);
        
        //listaLocaisEntrega é a lista usada para gerar o dataView da primeira tabela, a de locais de entrega.
        panelItensLicitacao.setListaLocaisEntrega(new ArrayList<BemUfDto>());
        
        //Atualiza os contadores. Sempre que forem adicionados novos itens e grupos o contador deverá iniciar em 1
        panelItensLicitacao.setContadorGruposAdicionados(1);
        panelItensLicitacao.setContadorItensAdicionados(1);
    }
    
    public void atualizarPainelLocaisDeEntrega(AjaxRequestTarget target){
        
        listaLocaisEntrega = inscricaoService.buscarListaBemUfDto(form.getModelObject().getPrograma());
        copiarConteudoDaListaDeLocaisDeEntrega();
        
        //A lista original sempre será usada quando um item ou grupo for excluido, baseado nela vou voltar a primeira tabela aos valores originais.
        panelItensLicitacao.setListaComDadosOriginaisBemUfDto(listaComDadosOriginaisBemUfDto);
        
        //listaLocaisEntrega é a lista usada para gerar o dataView da primeira tabela, a de locais de entrega.
        panelItensLicitacao.setListaLocaisEntrega(listaLocaisEntrega);
        
        //Atualiza os contadores. Sempre que forem adicionados novos itens e grupos o contador deverá iniciar em 1
        panelItensLicitacao.setContadorGruposAdicionados(1);
        panelItensLicitacao.setContadorItensAdicionados(1);
        
        panelItensLicitacao.getPanelPrimeiro().addOrReplace(panelItensLicitacao.newDataViewPrimeiraLista());
        panelItensLicitacao.getPanelPrimeiro().addOrReplace(panelItensLicitacao.newPaginatorPrimeiraLista());
        target.add(panelItensLicitacao);
    }
    
    private void copiarConteudoDaListaDeLocaisDeEntrega(){
        
        listaComDadosOriginaisBemUfDto=new ArrayList<BemUfDto>();
        
        for(BemUfDto bem:listaLocaisEntrega){
            
            BemUfDto novo = new BemUfDto();
            novo.setBem(bem.getBem());
            novo.setQuantidade(bem.getQuantidade());
            novo.setUf(bem.getUf());
            novo.setSelecionado(false);
            
            listaComDadosOriginaisBemUfDto.add(novo);
        }
    }
    
    private void actionVoltar() {
        setResponsePage(backPage);
    }
    
    private void actionSalvar()
    {
        if(!validarCampos()){
            panelItensLicitacao.setPrimeiroLoop(true);
            return;
        }
        
        List<AgrupamentoLicitacao> listaPronta = montarListaAgrupamentos();
        
        //Irá verificar se todos os inputs dos Grupos e Itens foram preenchidos
        if(!validarSeTodosInputsDaListaForamDigitados(listaPronta)){
            return;
        }
        
        licitacaoPrograma.setListaAgrupamentoLicitacao(listaPronta);
        licitacaoService.incluirAlterar(licitacaoPrograma, getIdentificador());
        
        if(licitacaoPrograma.getId() == null){
            getSession().info("Licitação salva com sucesso.");
        }
        else{
            getSession().info("Licitação editada com sucesso.");
        }
        setResponsePage(new PlanejamentoLicitacaoPesquisaPage(getPageParameters()));
    }
    
    private List<AgrupamentoLicitacao> montarListaAgrupamentos(){
         List<AgrupamentoLicitacao> agrupamento = montarListaDeItens();
         List<AgrupamentoLicitacao> listaPronta = montarListaDeGrupos(agrupamento);
         return listaPronta;
    }
    
    private List<AgrupamentoLicitacao> montarListaDeItens(){
        List<AgrupamentoLicitacao> agrupamento = new ArrayList<AgrupamentoLicitacao>();
        
        List<ItemBemDto> listaItens = new ArrayList<ItemBemDto>();
        listaItens = panelItensLicitacao.getListaDeItens();
        for(ItemBemDto item:listaItens){
            
            AgrupamentoLicitacao ag1 = new AgrupamentoLicitacao();
            ag1.setNomeAgrupamento(item.getNomeModelDto().getNomeGrupo());
            ag1.setTipoAgrupamentoLicitacao(item.getTipoAgrupamentoLicitacao());
            ag1.setId(item.getAgrupamentoLicitacao() != null?item.getAgrupamentoLicitacao().getId():null);
            
            if(item.getAgrupamentoLicitacao() != null){
                ag1.setLicitacaoPrograma(licitacaoPrograma!=null?licitacaoPrograma:null);
            }
            
            List<SelecaoItem> listaSelecao = new ArrayList<SelecaoItem>();
            SelecaoItem selecao = new SelecaoItem();
            
            String unidadeMedida = item.getUnidadeDeMedida();
            Long quantidadeRegistrar = item.getQuantidadeRegistrar();
            BigDecimal valorUnitario = item.getValorUnitario();
            BigDecimal valorTotal = item.getValorTotal();
            
            selecao.setUnidadeMedida(unidadeMedida);
            selecao.setQuantidadeRegistrar(quantidadeRegistrar);
            selecao.setValorUnitario(valorUnitario);
            selecao.setValorTotal(valorTotal);
            selecao.setAgrupamentoLicitacao(item.getAgrupamentoLicitacao()!=null?item.getAgrupamentoLicitacao():null);
            
            List<BemUf> listaUf = new ArrayList<BemUf>();
            
            for(BemUfDto item2:item.getListaDeBens()){
                
                BemUf bemUf = new BemUf();
                bemUf.setBem(item2.getBem());
                bemUf.setQuantidade(item2.getQuantidade());
                bemUf.setUf(item2.getUf());
                bemUf.setSelecaoItem(item2.getSelecaoItem());
                bemUf.setId(item2.getId()!=null?item2.getId():null);
                
                if(item2.getSelecaoItem()==null){
                    
                }
                else{
                    selecao.setId(item2.getSelecaoItem().getId());
                }
                
                listaUf.add(bemUf);
            }
            
            selecao.setListaBemUf(listaUf);
            listaSelecao.add(selecao);
            ag1.setListaSelecaoItem(listaSelecao);
            
            agrupamento.add(ag1);
        }
        return agrupamento;
    }
    
    private List<AgrupamentoLicitacao> montarListaDeGrupos(List<AgrupamentoLicitacao> agrupamento ){
      //Montando os Grupos
        List<ItemBemDto> listaGrupos = new ArrayList<ItemBemDto>();
        listaGrupos = panelItensLicitacao.getListaDeGrupos();
        
        List<AgrupamentoLicitacao> listaAgrupamentot = new ArrayList<AgrupamentoLicitacao>();
        Integer numeroTemp = 0;
        AgrupamentoLicitacao ag1 = new AgrupamentoLicitacao();
        int flag = 0;
        
        for(ItemBemDto item:listaGrupos){
            
            if(numeroTemp != item.getContadorTemp()){
                numeroTemp = item.getContadorTemp();
                
                //Ira entrar neste 'if' somente depois que for criado a primeira instancia de AgrupamentoLicitação
                if(flag > 0){
                    agrupamento.add(ag1);
                }
                
                ag1 = new AgrupamentoLicitacao();
                ag1.setNomeAgrupamento(item.getNomeModelDto().getNomeGrupo());
                ag1.setTipoAgrupamentoLicitacao(item.getTipoAgrupamentoLicitacao());
                ag1.setId(item.getAgrupamentoLicitacao() != null?item.getAgrupamentoLicitacao().getId():null);
                
                if(item.getAgrupamentoLicitacao()!=null){
                    ag1.setLicitacaoPrograma(licitacaoPrograma!=null?licitacaoPrograma:null);
                }
                
                flag++;
            }

            SelecaoItem selecao = new SelecaoItem();
            
            String unidadeMedida = item.getUnidadeDeMedida();
            Long quantidadeRegistrar = item.getQuantidadeRegistrar();
            BigDecimal valorUnitario = item.getValorUnitario();
            BigDecimal valorTotal = item.getValorTotal();
            
            selecao.setUnidadeMedida(unidadeMedida);
            selecao.setQuantidadeRegistrar(quantidadeRegistrar);
            selecao.setValorUnitario(valorUnitario);
            selecao.setValorTotal(valorTotal);
            selecao.setAgrupamentoLicitacao(item.getAgrupamentoLicitacao()!=null?item.getAgrupamentoLicitacao():null);
            
            List<BemUf> listaUf = new ArrayList<BemUf>();
            
            for(BemUfDto item2:item.getListaDeBens()){
                
                BemUf bemUf = new BemUf();
                bemUf.setBem(item2.getBem());
                bemUf.setQuantidade(item2.getQuantidade());
                bemUf.setUf(item2.getUf());
                bemUf.setSelecaoItem(item2.getSelecaoItem());
                bemUf.setId(item2.getId()!=null?item2.getId():null);
                
                if(item2.getSelecaoItem()==null){
                    
                }
                else{
                    selecao.setId(item2.getSelecaoItem().getId());
                }
                
                listaUf.add(bemUf);
            }
            
            selecao.setListaBemUf(listaUf);
            ag1.getListaSelecaoItem().add(selecao);
        }
        
        //Insere o ultimo grupo;
        if(flag>0){
            agrupamento.add(ag1);
        }
        return agrupamento;
    }
    
    
    private boolean validarCampos(){
        boolean validar = true;
        licitacaoPrograma = form.getModelObject();
        
        validar = validarInformacoesBasicas(validar,licitacaoPrograma);
        validar = validarSeTodosItensForamEscolhidos(validar);
        
        return validar;
    }
    
    private boolean validarInformacoesBasicas(boolean validar, LicitacaoPrograma licitacaoPrograma){
        
        if(licitacaoPrograma.getPrograma() == null || licitacaoPrograma.getPrograma().getId() == null)
        {
            addMsgError("O campo 'Programa' é obrigatório.");
            validar = false;
        }
        
        if(licitacaoPrograma.getObjeto() == null || licitacaoPrograma.getObjeto().length() == 0)
        {
            addMsgError("O campo 'Objeto' é obrigatório.");
            validar = false;
        }
        
        if(licitacaoPrograma.getJustificativa() == null || licitacaoPrograma.getJustificativa().length() == 0)
        {
            addMsgError("O campo 'Justificativa' é obrigatório.");
            validar = false;
        }
        
        if(licitacaoPrograma.getEspecificacoesEQuantidadeDoObjeto() == null || licitacaoPrograma.getEspecificacoesEQuantidadeDoObjeto().length() == 0)
        {
            addMsgError("O campo 'Especificações e Quantidade do Objeto' é obrigatório.");
            validar = false;
        }
        
        if(licitacaoPrograma.getRecebimentoEAceitacaoDosMateriais() == null || licitacaoPrograma.getRecebimentoEAceitacaoDosMateriais().length() == 0)
        {
            addMsgError("O campo 'Recebimento e Aceitação dos Materiais' é obrigatório.");
            validar = false;
        }
        
        if(licitacaoPrograma.getPrazoLocalEFormaDeEntrega() == null || licitacaoPrograma.getPrazoLocalEFormaDeEntrega().length() == 0)
        {
            addMsgError("O campo 'Prazo, Local e Forma de Entrega' é obrigatório.");
            validar = false;
        }
        
        if(licitacaoPrograma.getMetodologiaDeAvaliacaoEAceiteDosMateriais() == null || licitacaoPrograma.getMetodologiaDeAvaliacaoEAceiteDosMateriais().length() == 0)
        {
            addMsgError("O campo 'Metodologia de Avaliação e Aceite dos Materiais' é obrigatório.");
            validar = false;
        }
        
        if(licitacaoPrograma.getDataInicialPeriodoExecucao() == null)
        {
            addMsgError("A data inicial de execução é obrigatória.");
            validar = false;
        }
        
        if(licitacaoPrograma.getDataFinalPeriodoExecucao() == null)
        {
            addMsgError("A data final de execução é obrigatória.");
            validar = false;
        }
        
        if(licitacaoPrograma.getDataInicialPeriodoExecucao()!=null && licitacaoPrograma.getDataFinalPeriodoExecucao()!=null){
            if(licitacaoPrograma.getDataInicialPeriodoExecucao().isAfter(licitacaoPrograma.getDataFinalPeriodoExecucao())){
                addMsgError("A data inicial de execução não pode ser superior a data final de execução.");
                validar = false;
            }
        }   
        return validar;
    }
    
    private boolean validarSeTodosItensForamEscolhidos(boolean validar){
        if(panelItensLicitacao.getListaLocaisEntrega().size() != 0){
            addMsgError("Todos os itens da tabela 'Locais de Entrega' deverão ser selecionados.");
            validar = false;
        }
        
        return validar;
    }
    
    private boolean validarSeExistemGruposComMesmoNome(boolean validar){
        
        //Verificar se Existem Grupos com o mesmo nome
        List<ItemBemDto> lista = panelItensLicitacao.getListaDeGrupos();
        for(ItemBemDto ibd:lista){
            String nomeLoop = ibd.getNomeModelDto().getNomeGrupo();
            Integer idTemp = ibd.getContadorTemp();

            for(ItemBemDto ibd2:lista){
                String nomeLoop2 = ibd2.getNomeModelDto().getNomeGrupo();
                Integer idTemp2 = ibd2.getContadorTemp();
                
                if(nomeLoop.equalsIgnoreCase(nomeLoop2) && idTemp!=idTemp2){
                    addMsgError("Não é possível cadastrar 2 grupos com o mesmo nome.");
                    return  false;
                }
            }
        }        
        return validar;
    }
    
    private boolean validarSeExistemItensComMesmoNome(boolean validar){
        
      //Verificar se Existem Grupos com o mesmo nome
        List<ItemBemDto> lista = panelItensLicitacao.getListaDeItens();
        for(ItemBemDto ibd:lista){
            String nomeLoop = ibd.getNomeModelDto().getNomeGrupo();
            Integer idTemp = ibd.getContadorTemp();

            for(ItemBemDto ibd2:lista){
                String nomeLoop2 = ibd2.getNomeModelDto().getNomeGrupo();
                Integer idTemp2 = ibd2.getContadorTemp();
                
                if(nomeLoop.equalsIgnoreCase(nomeLoop2) && idTemp!=idTemp2){
                    addMsgError("Não é possível cadastrar 2 itens com o mesmo nome.");
                    return  false;
                }
            }
        }        
        return validar;
    }
    
    private boolean validarSeTodosInputsDaListaForamDigitados(List<AgrupamentoLicitacao> listaPronta){
        boolean validar = true;
        int qtdItensVazios = 0;
        int qtdGruposVazios = 0;
        
        for(AgrupamentoLicitacao agrup:listaPronta){
            if(agrup.getNomeAgrupamento() == null || "".equalsIgnoreCase(agrup.getNomeAgrupamento())){
                if(agrup.getTipoAgrupamentoLicitacao() == EnumTipoAgrupamentoLicitacao.GRUPO){
                    qtdGruposVazios ++;
                }else{
                    qtdItensVazios ++;
                }
            }
            
            for(SelecaoItem si:agrup.getListaSelecaoItem()){
                if(agrup.getTipoAgrupamentoLicitacao() == EnumTipoAgrupamentoLicitacao.GRUPO){
                    qtdGruposVazios += verificarCampoNuloGrupo(si);
                }else{
                    qtdItensVazios += verificarCampoNuloItem(si);
                }
            }
        }    

        if(qtdGruposVazios>0){
            addMsgError("Existe(m) "+qtdGruposVazios+" campo(s) vazio(s) ou com valores '0' na tabela 'Grupos Selecionados'.");
            validar = false;
        }
        
        if(qtdItensVazios>0){
            addMsgError("Existe(m) "+qtdItensVazios+" campo(s) vazio(s) ou com valores '0' na tabela 'Itens Selecionados'.");
            validar = false;
        }
        
        return validar;
    }
    
    private Integer verificarCampoNuloItem(SelecaoItem si){
        int vazios = 0;
        
        if(si.getUnidadeMedida()==null || "".equalsIgnoreCase(si.getUnidadeMedida())){
            vazios ++;
        }
        
        if(si.getQuantidadeRegistrar()==null || si.getQuantidadeRegistrar() == 0){
            vazios ++;
        }
        
        if(si.getValorUnitario()==null || si.getValorUnitario().doubleValue() == 0){
            vazios ++;
        }
        
        if(si.getValorTotal()==null || si.getValorTotal().doubleValue() == 0){
            vazios ++;
        }
        
        return vazios;
    }
    
    private Integer verificarCampoNuloGrupo(SelecaoItem si){
        int vazios = 0;
        
        if(si.getUnidadeMedida()==null || "".equalsIgnoreCase(si.getUnidadeMedida())){
            vazios ++;
        }
        
        if(si.getQuantidadeRegistrar()==null || si.getQuantidadeRegistrar() == 0){
            vazios ++;
        }
        
        if(si.getValorUnitario()==null || si.getValorUnitario().doubleValue() == 0){
            vazios ++;
        }
        
        if(si.getValorTotal()==null || si.getValorTotal().doubleValue() == 0){
            vazios ++;
        }
        
        return vazios;
    }
    
    private boolean botoesEnabled(){
        if(readOnly){
            return false;
        }else{
            return true;
        }
        
    }
    
    private String limparNumeroSei(String valor) {
        if (valor == null || "".equalsIgnoreCase(valor)) {
            return "";
        }
        String value = valor;
        value = value.replace(".", "");
        value = value.replace("/", "");
        value = value.replace("-", "");
        value = value.replace("R$", "");
        value = value.replace(" ", "");
        value = value.replace(",", ".");
        return value;
    }
    
    private boolean dropProgramaEnabled(){
        if(readOnly || (licitacaoPrograma != null && licitacaoPrograma.getId() != null)){
            return false;
        }else{
            return true;
        }
    }

    private boolean mostrarBotaoTrocarLicitacao(){
        if(readOnly){
            return false;
        }else{
            if(licitacaoPrograma == null || licitacaoPrograma.getId() == null){
                return false;
            }else{
                return true;
            }
        }
    }
    
    private String gerarTituloPagina(){
        String tituloPagina="";
        if(readOnly){
            tituloPagina = "Visualizar Licitação";
        }else{
            if(licitacaoPrograma == null || licitacaoPrograma.getId() == null){
                tituloPagina = "Planejar Licitação";
            }else{
                tituloPagina = "Editar Licitação";
            }
        }
        return tituloPagina;
    }

    public String getMsgConfirm() {
        return msgConfirm;
    }


    public void setMsgConfirm(String msgConfirm) {
        this.msgConfirm = msgConfirm;
    }
}
