package br.gov.mj.side.web.view.programa;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;

import javax.inject.Inject;

import org.apache.wicket.Page;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.authroles.authorization.strategies.role.annotations.AuthorizeInstantiation;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Button;
import org.apache.wicket.markup.html.form.ChoiceRenderer;
import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.cycle.RequestCycle;
import org.apache.wicket.request.handler.resource.ResourceRequestHandler;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.validation.validator.StringValidator;
import org.wicketstuff.jasperreports.JRConcreteResource;
import org.wicketstuff.jasperreports.handlers.PdfResourceHandler;

import br.gov.mj.apoio.entidades.Funcao;
import br.gov.mj.apoio.entidades.Orgao;
import br.gov.mj.apoio.entidades.SubFuncao;
import br.gov.mj.apoio.entidades.UnidadeExecutora;
import br.gov.mj.infra.wicket.component.ComponentFactory;
import br.gov.mj.side.entidades.enums.EnumStatusPrograma;
import br.gov.mj.side.entidades.programa.Programa;
import br.gov.mj.side.entidades.programa.ProgramaBem;
import br.gov.mj.side.entidades.programa.ProgramaKit;
import br.gov.mj.side.entidades.programa.ProgramaPotencialBeneficiarioUf;
import br.gov.mj.side.web.service.OrgaoService;
import br.gov.mj.side.web.service.ProgramaService;
import br.gov.mj.side.web.service.PublicizacaoService;
import br.gov.mj.side.web.service.UnidadeExecutoraService;
import br.gov.mj.side.web.util.RelatorioConsultaPublicaBuilder;
import br.gov.mj.side.web.view.HomePage;
import br.gov.mj.side.web.view.programa.anexo.PanelAnexoPage;
import br.gov.mj.side.web.view.programa.criterioAcompanhamento.PanelCriterioAcompanhamentoPage;
import br.gov.mj.side.web.view.programa.criterioElegibilidade.PanelCriterioElegibilidadePage;
import br.gov.mj.side.web.view.programa.criterioavaliacao.CriterioAvaliacaoProgramaPanel;
import br.gov.mj.side.web.view.programa.informacoesGerais.PanelInformacoesGeraisPage;
import br.gov.mj.side.web.view.programa.potenciaisBeneficiarios.PanelPotenciaisBeneficiariosPage;
import br.gov.mj.side.web.view.programa.publicizacao.HistoricoProgramaPanel;
import br.gov.mj.side.web.view.programa.recursofinanceiro.RecursoFinanceiroPanel;
import br.gov.mj.side.web.view.programa.vincularBemKit.PanelVincularBemKit;
import br.gov.mj.side.web.view.programa.visualizacao.ProgramaAnexosPublicadosPanel;
import br.gov.mj.side.web.view.template.TemplatePage;

@AuthorizeInstantiation({ ProgramaPesquisaPage.ROLE_MANTER_PROGRAMA_INCLUIR, ProgramaPesquisaPage.ROLE_MANTER_PROGRAMA_ALTERAR, ProgramaPesquisaPage.ROLE_MANTER_PROGRAMA_EXCLUIR, ProgramaPesquisaPage.ROLE_MANTER_PROGRAMA_VISUALIZAR, ProgramaPesquisaPage.ROLE_MANTER_PROGRAMA_PUBLICAR })
public class ProgramaPage extends TemplatePage {

    private static final long serialVersionUID = 1L;
    public static final String BUSCA_PADRAO = "MJ";

    private Form<Programa> form;
    private List<ProgramaKit> listaKitsVinculados;
    private Programa entity;
    private boolean readOnly;
    private Page backPage;

    private Orgao orgao;
    private UnidadeExecutora unidadeExecutora;
    
    private PanelInformacoesGeraisPage panelInformacoesGerais;
    private RecursoFinanceiroPanel recursoFinanceiroPanel;
    private PanelPotenciaisBeneficiariosPage panelPotenciaisBeneficiarios;
    private PanelOrgaoExecutor panelOrgaoExecutor;
    private PanelVincularBemKit panelVincularBemKit;
    private PanelCriterioElegibilidadePage panelCriterioElegibilidade;
    private PanelCriterioAcompanhamentoPage panelCriterioAcompanhamento;
    private CriterioAvaliacaoProgramaPanel criterioAvaliacaoPanel;
    private PanelAnexoPage panelAnexoPage;
    private PanelBotoes panelBotoes;

    private Button buttonSalvar;
    private Button buttonFinalizar;

    @Inject
    private ComponentFactory componentFactory;

    @Inject
    private OrgaoService orgaoService;

    @Inject
    private UnidadeExecutoraService unidadeExecutoraService;

    @Inject
    private ProgramaService programaService;

    @Inject
    private PublicizacaoService publicizacaoService;

    public ProgramaPage(final PageParameters pageParameters) {
        super(pageParameters);
        inicializarVariaveis();
        initComponents();
        criarBreadcrumb();
    }

    public ProgramaPage(final PageParameters pageParameters, Page backPage, Programa programa, boolean readOnly) {
        super(pageParameters);

        this.backPage = backPage;

        if (programa != null && programa.getId() != null) {
            entity = programaService.buscarPeloId(programa.getId());
            entity.setCriteriosAcompanhamento(programaService.buscarProgramaCriterioAcompanhamento(programa));
            entity.setCriteriosElegibilidade(programaService.buscarProgramaCriterioElegibilidade(programa));
            entity.setProgramaBens(programaService.buscarProgramaBem(programa));
            entity.setProgramaKits(programaService.buscarProgramakit(programa));
            entity.setPotenciaisBeneficiariosUf(programaService.buscarProgramaPotencialBeneficiarioUf(programa));
            entity.setRecursosFinanceiros(programaService.buscarProgramaRecursoFinanceiroPeloIdPrograma(programa));
            entity.setHistoricoPublicizacao(programaService.buscarHistoricoPublicizacao(programa));
            entity.setCriteriosAvaliacao(programaService.buscarProgramaCriterioAvaliacao(programa));

        } else {
            entity = programa;
        }

        this.readOnly = readOnly;

        inicializarVariaveis();
        initComponents();
        criarBreadcrumb();

        if (readOnly) {
            panelAnexoPage.getDrop().setEnabled(false);
            panelAnexoPage.getButtonAdicionar().setEnabled(false);
            panelAnexoPage.getFormUpload().setEnabled(false);

            panelCriterioAcompanhamento.setEnabled(false);
            panelCriterioElegibilidade.setEnabled(false);
            criterioAvaliacaoPanel.setEnabled(false);
            panelInformacoesGerais.setEnabled(false);
            panelOrgaoExecutor.setEnabled(false);
            panelPotenciaisBeneficiarios.setEnabled(false);
            panelVincularBemKit.setEnabled(false);

            buttonFinalizar.setEnabled(false);
            buttonSalvar.setEnabled(false);
            setTitulo("Visualizar Programa");
        } else {
            if (entity.getId() == null) {
                setTitulo("Cadastrar Programa");
            } else {
                setTitulo("Editar Programa");
            }
        }
    }

    private void initComponents() {
        form = componentFactory.newForm("form", new CompoundPropertyModel<Programa>(entity));
        add(form);

        panelVincularBemKit = new PanelVincularBemKit("panelVincularBemKit", this);
        form.add(panelVincularBemKit);
        
        criterioAvaliacaoPanel = new CriterioAvaliacaoProgramaPanel("panelCriterioAvaliacao", this);
        form.add(criterioAvaliacaoPanel);
        
        panelCriterioElegibilidade = new PanelCriterioElegibilidadePage("panelCriterioElegibilidade", this);
        form.add(panelCriterioElegibilidade);

        panelCriterioAcompanhamento = new PanelCriterioAcompanhamentoPage("panelCriterioAcompanhamento", this);
        form.add(panelCriterioAcompanhamento);

        panelInformacoesGerais = new PanelInformacoesGeraisPage("panelInformacoesGerais", this);
        form.add(panelInformacoesGerais);

        recursoFinanceiroPanel = new RecursoFinanceiroPanel("recursoFinanceiroPanel", this, readOnly);
        form.add(recursoFinanceiroPanel);

        panelPotenciaisBeneficiarios = new PanelPotenciaisBeneficiariosPage("panelPotenciaisBeneficiarios", this);
        form.add(panelPotenciaisBeneficiarios);

        panelOrgaoExecutor = new PanelOrgaoExecutor("panelOrgaoExecutor");
        form.add(panelOrgaoExecutor);

        panelAnexoPage = new PanelAnexoPage("panelAnexoPage", this, isReadOnly());
        form.add(panelAnexoPage);

        form.add(getTextFieldNumeroSei()); // numeroProcessoSEI
        form.add(newLabelCriacaoUsuario());

        HistoricoProgramaPanel historicoPanel = new HistoricoProgramaPanel("historicoPanel", getEntity());
        historicoPanel.setVisible(isCreateMode());
        form.add(historicoPanel);
        
        ProgramaAnexosPublicadosPanel programaAnexosPublicadosPanel = new ProgramaAnexosPublicadosPanel("programaAnexosPublicadosPanel", getEntity());
        programaAnexosPublicadosPanel.setVisible(isCreateMode());
        
        form.add(programaAnexosPublicadosPanel);

        // Painel somente com os botões para salvar, finalizar e cancelar.
        panelBotoes = new PanelBotoes("panelBotoes");
        form.add(panelBotoes);
    }

    private boolean isCreateMode() {
        return getEntity().getId()!=null?true:false;
    }

    private class PanelOrgaoExecutor extends WebMarkupContainer {
        public PanelOrgaoExecutor(String id) {
            super(id);

            add(getDropDownOrgao()); // nomeOrgao
            add(getDropDownUnidadeExecutora()); // unidadeExecutora
        }
    }

    private class PanelBotoes extends WebMarkupContainer {
        public PanelBotoes(String id) {
            super(id);

            add(getButtonSalvar()); // btnSalvar
            add(getButtonFinalizar()); // btnFinalizar
            add(getButtonVoltar()); // btnCancelar
            add(getButtonRelatorio()); // btnPdf

            Button btnPublicar = getButtonPublicar();
            authorize(btnPublicar, RENDER, ProgramaPesquisaPage.ROLE_MANTER_PROGRAMA_PUBLICAR);
            add(btnPublicar);
        }
    }

    private void criarBreadcrumb() {
        form.add(new Link("dashboard") {

            @Override
            public void onClick() {
                setResponsePage(HomePage.class);
            }
        });

        form.add(new Link("programaPage") {

            @Override
            public void onClick() {
                setResponsePage(ProgramaPesquisaPage.class);
            }
        });
    }

    public void inicializarVariaveis() {
        if (listaKitsVinculados == null) {
            listaKitsVinculados = new ArrayList<ProgramaKit>();
        }

        if (entity != null && entity.getId() != null) {
            orgao = entity.getUnidadeExecutora().getOrgao();
            unidadeExecutora = entity.getUnidadeExecutora();
        } else {

            orgao = new Orgao();
            unidadeExecutora = new UnidadeExecutora();
        }
    }

    /*
     * OS COMPONENTES SERÃO IMPLEMENTADOS ABAIXO
     */

    public DropDownChoice<Orgao> getDropDownOrgao() {

        List<Orgao> listOrgao = orgaoService.buscarTodos();
        for (Orgao result : listOrgao) {
            String sigla = result.getSiglaOrgao();
            if (sigla.equals(BUSCA_PADRAO)) {
                orgao = result;
                break;
            }
        }

        DropDownChoice<Orgao> drop = new DropDownChoice<Orgao>("nomeOrgao", new PropertyModel<Orgao>(this, "orgao"), listOrgao, new ChoiceRenderer<Orgao>("codigoOrgaoSigla", "id"));
        drop.setNullValid(true);
        drop.setOutputMarkupId(true);
        actionDropDownOrgao(drop);
        return drop;
    }

    public DropDownChoice<UnidadeExecutora> getDropDownUnidadeExecutora() {

        List<UnidadeExecutora> listUnidadeExecutora = new ArrayList<UnidadeExecutora>();
        if (orgao != null && orgao.getId() != null) {
            listUnidadeExecutora = unidadeExecutoraService.buscarPeloOrgaoId(orgao.getId());
        }
        DropDownChoice<UnidadeExecutora> drop = new DropDownChoice<UnidadeExecutora>("unidadeExecutora", new PropertyModel<UnidadeExecutora>(this, "unidadeExecutora"), listUnidadeExecutora, new ChoiceRenderer<UnidadeExecutora>("codigoNome", "id"));
        actionDropDownUnidadeExecutora(drop);
        drop.setOutputMarkupId(true);
        drop.setNullValid(true);
        return drop;
    }

    public TextField<String> getTextFieldNumeroSei() {
        return componentFactory.newTextField("numeroProcessoSEI", "Número Processo SEI", false, null, StringValidator.maximumLength(20));
    }

    private Label newLabelCriacaoUsuario() {
        String frase = "";
        String usuario="";
        if (entity.getDataAlteracao() == null) {
            frase += "<strong>Data de criação do programa: </strong>" + dataCadastroBR(entity.getDataCadastro());
            usuario+=entity.getUsuarioCadastro();
        } else {
            frase += "<strong>Data da última alteração: </strong>" + dataCadastroBR(entity.getDataAlteracao());
            usuario+=entity.getUsuarioAlteracao();
        }
        
        Label label = new Label("lblUsuario", frase + ", <strong>usuário: </strong>" + usuario);
        label.setEscapeModelStrings(false);
        label.setVisible(entity.getId() != null);
        return label;
    }

    public String dataCadastroBR(LocalDateTime dataCadastro) {
        DateTimeFormatter sdfPadraoBR = DateTimeFormatter.ofPattern("dd/MM/yyyy HH:mm:ss");
        if (dataCadastro != null) {
            return sdfPadraoBR.format(dataCadastro);
        }
        return " - ";
    }

    private Button getButtonSalvar() {
        buttonSalvar = componentFactory.newButton("btnSalvar", () -> salvarPrograma());
        if (entity.getStatusPrograma() != null && !entity.getStatusPrograma().equals(EnumStatusPrograma.EM_ELABORACAO)) {
            buttonSalvar.setVisible(false);
        }
        return buttonSalvar;
    }

    private Button getButtonFinalizar() {
        buttonFinalizar = componentFactory.newButton("btnFinalizar", () -> finalizarPrograma());
        if (getEntity().getStatusPrograma() != null && !getEntity().getStatusPrograma().equals(EnumStatusPrograma.EM_ELABORACAO) && !getEntity().getStatusPrograma().equals(EnumStatusPrograma.FORMULADO)) {
            buttonFinalizar.setVisible(false);
        }
        return buttonFinalizar;
    }

    private Button getButtonPublicar() {
        Button button = componentFactory.newButton("btnRepublicar", () -> republicar());

        if (getEntity().getStatusPrograma() != null && getEntity().getStatusPrograma().equals(EnumStatusPrograma.PUBLICADO)) {
            button.setVisible(true);
        } else {
            button.setVisible(false);
        }
        return button;
    }

    private void republicar() {
        if (!getSideSession().hasRole(ProgramaPesquisaPage.ROLE_MANTER_PROGRAMA_PUBLICAR)) {
            throw new SecurityException();
        }
        getEntity().setNumeroProcessoSEI(limparNumeroSei(getEntity().getNumeroProcessoSEI()));
        publicizacaoService.republicar(getEntity(), getIdentificador());
        getSession().info("Publicado com sucesso");
        setResponsePage(new ProgramaPesquisaPage(null));
    }

    private Button getButtonRelatorio() {
        Button buttonCsv = componentFactory.newButton("btnPdf", () -> gerarPdf());
        buttonCsv.setVisible(entity.getId() != null);
        return buttonCsv;
    }

    private Button getButtonVoltar() {
        Button button = componentFactory.newButton("btnVoltar", () -> voltar());
        button.setDefaultFormProcessing(false);
        return button;
    }

    /*
     * AQUI VIRÃO AS AÇÕES
     */

    // Metodo chamado quando for digitada uma data, as ações orçamentárias serão
    // filtradas pelo ano
    // Esta chamada é feita a partir do PanelInformacoesGerais quando digita-se
    // o ano.
    public void atualizarDropDownRecursoFinanceiro(AjaxRequestTarget target) {
        recursoFinanceiroPanel.addOrReplace(recursoFinanceiroPanel.newDropDownAcaoOrcamentaria());
        target.add(recursoFinanceiroPanel);
    }
    
    public void removerRecursosFinanceiros(AjaxRequestTarget target){
        getForm().getModelObject().getRecursosFinanceiros().clear();
        recursoFinanceiroPanel.atualizarLabelTotal();
        atualizarDropDownRecursoFinanceiro(target);
        target.add(recursoFinanceiroPanel);
    }
    

    public void salvarPrograma() {

        if (!getSideSession().hasAnyRole(new String[] { ProgramaPesquisaPage.ROLE_MANTER_PROGRAMA_INCLUIR, ProgramaPesquisaPage.ROLE_MANTER_PROGRAMA_ALTERAR })) {
            throw new SecurityException();
        }
        panelPotenciaisBeneficiarios.nivelarListTempUfselected();

        Programa programaTemp = new Programa();
        programaTemp = form.getModelObject();
        programaTemp.setStatusPrograma(EnumStatusPrograma.EM_ELABORACAO);
        String numeroSei = limparNumeroSei(programaTemp.getNumeroProcessoSEI());
        programaTemp.setNumeroProcessoSEI(numeroSei);
        programaTemp.setDataCadastro(LocalDateTime.now());

        boolean editando = programaTemp.getId() == null ? false : true;

        if (!validarSalvar(programaTemp)) {
            return;
        }

        String numero = limparNumeroSei(programaTemp.getNumeroProcessoSEI());
        programaTemp.setNumeroProcessoSEI(numero);
        String usuarioLogado = getIdentificador();
        programaTemp.setUsuarioAlteracao(usuarioLogado);
        programaService.incluirAlterar(programaTemp, usuarioLogado);

        if (editando) {
            getSession().info("Alterado com sucesso");
        } else {
            getSession().info("Salvo com sucesso.");
        }
        setResponsePage(new ProgramaPesquisaPage(null));

    }

    public void finalizarPrograma() {
        Programa programaTemp = new Programa();
        programaTemp = form.getModelObject();
        programaTemp.setStatusPrograma(EnumStatusPrograma.FORMULADO);

        String numeroSei = limparNumeroSei(programaTemp.getNumeroProcessoSEI());
        programaTemp.setNumeroProcessoSEI(numeroSei);
        boolean editando = programaTemp.getId() == null ? false : true;

        if (!validarFinalizar(programaTemp)) {
            return;
        }

        String usuarioLogado = getIdentificador();
        programaService.incluirAlterar(programaTemp, usuarioLogado);

        if (editando) {
            getSession().info("Alterado com sucesso");
        } else {
            getSession().info("Finalizado com sucesso.");
        }
        setResponsePage(new ProgramaPesquisaPage(null));
    }

    private void gerarPdf() {

        List<Programa> lista = new ArrayList<Programa>();
        lista.add(entity);
        RelatorioConsultaPublicaBuilder builder = new RelatorioConsultaPublicaBuilder(programaService);
        JRConcreteResource<PdfResourceHandler> relatorioResource = builder.export(lista);
        ResourceRequestHandler handler = new ResourceRequestHandler(relatorioResource, getPageParameters());
        RequestCycle requestCycle = getRequestCycle();
        requestCycle.scheduleRequestHandlerAfterCurrent(handler);
    }

    public void voltar() {
        setResponsePage(backPage);
    }

    public boolean validarSalvar(Programa programa) {
        boolean validar = true;

        validar = validarNomePrograma(programa, validar);
        validar = validarNomeFantasia(programa, validar);
        validar = validarDescricao(programa, validar);
        validar = validarAnoPrograma(programa, validar);
        validar = validarFuncao(validar);
        validar = validarSubFuncao(validar);
        validar = validarOrgao(validar);
        validar = validarUnidadeExecutora(programa, validar);
        validar = validarLimitacaoGeografica(programa, validar);
        validar = validarNomePersonalidadeJuridica(programa, validar);
        validar = validarNumeroProcessoSei(programa, validar);
        return validar;
    }

    public boolean validarFinalizar(Programa programa) {
        boolean validar = true;

        validar = validarNomePrograma(programa, validar);
        validar = validarNomeFantasia(programa, validar);
        validar = validarDescricao(programa, validar);
        validar = validarAnoPrograma(programa, validar);
        validar = validarFuncao(validar);
        validar = validarSubFuncao(validar);
        validar = validarRecursoFinanceiro(programa, validar);
        validar = validarOrgao(validar);
        validar = validarUnidadeExecutora(programa, validar);
        validar = validarLimitacaoGeografica(programa, validar);
        validar = validarPotenciaisBeneficiarios(programa, validar);
        validar = validarNomePersonalidadeJuridica(programa, validar);
        validar = validarBensKits(programa, validar);
        validar = validarCriterioElegibilidade(programa, validar);
        validar = validarCriterioAcompanhamento(programa, validar);
        validar = validarNumeroProcessoSei(programa, validar);
        validar = validarCriterioAvaliacao(programa,validar);
        
        return validar;
    }

    public boolean validarNomePrograma(Programa programa, boolean valor) {
        boolean validar = valor;

        if (programa.getNomePrograma() == null || "".equalsIgnoreCase(programa.getNomePrograma())) {
            addMsgError("O campo 'Nome do Programa' é obrigatório.");
            validar = false;
        }

        return validar;
    }

    public boolean validarNomeFantasia(Programa programa, boolean valor) {
        boolean validar = valor;

        if (programa.getNomeFantasiaPrograma() == null || "".equalsIgnoreCase(programa.getNomeFantasiaPrograma())) {
            addMsgError("O campo 'Nome Fantasia' é obrigatório.");
            validar = false;
        }

        return validar;
    }

    public boolean validarDescricao(Programa programa, boolean valor) {
        boolean validar = valor;

        if (programa.getDescricaoPrograma() == null || "".equalsIgnoreCase(programa.getDescricaoPrograma())) {
            addMsgError("O campo 'Descrição' é obrigatório.");
            validar = false;
        }

        return validar;
    }

    public boolean validarAnoPrograma(Programa programa, boolean valor) {
        boolean validar = valor;
        if (programa.getAnoPrograma() == null || programa.getAnoPrograma() == 0) {
            addMsgError("O campo 'Ano' é obrigatório.");
            validar = false;
        } else {
            validar = programa.getAnoPrograma().toString().length() == 4;
            if (!validar) {
                addMsgError("'Ano' não é um valor válido");
            }
        }
        return validar;
    }

    public boolean validarFuncao(boolean valor) {
        boolean validar = valor;

        Funcao func = panelInformacoesGerais.getFuncao();
        if (func == null || func.getId() == null) {
            addMsgError("O campo 'Função' é obrigatório.");
            validar = false;
        }

        return validar;
    }

    public boolean validarSubFuncao(boolean valor) {
        boolean validar = valor;

        SubFuncao sub = panelInformacoesGerais.getSubFuncao();
        if (sub == null || sub.getId() == null) {
            addMsgError("O campo 'Subfunção' é obrigatório.");
            validar = false;
        }

        return validar;
    }

    public boolean validarRecursoFinanceiro(Programa programa, boolean valor) {
        boolean validar = valor;

        if (programa.getRecursosFinanceiros() == null || programa.getRecursosFinanceiros().isEmpty()) {
            addMsgError("Vincule ao menos 1 Recurso Financeiro.");
            validar = false;
        }
        
        if(programa.getValorMaximoProposta().doubleValue() > recursoFinanceiroPanel.getValorTotal().doubleValue()){
            addMsgError("O Valor Máximo por Proposta não pode ser maior do que o Valor Total do Recurso Financeiro.");
            validar = false;
        }

        return validar;
    }

    public boolean validarCriterioAvaliacao(Programa programa, boolean valor) {
        boolean validar = valor;

        if (programa.getCriteriosAvaliacao() == null || programa.getCriteriosAvaliacao().isEmpty()) {
            addMsgError("Vincule ao menos 1 Critério de Avaliação.");
            validar = false;
        }

        return validar;
    }
    
    public boolean validarOrgao(boolean valor) {
        boolean validar = valor;

        if (orgao == null || orgao.getId() == null) {
            addMsgError("O campo 'Órgão (Cód. SIORG)' é obrigatório.");
            validar = false;
        }

        return validar;
    }

    public boolean validarUnidadeExecutora(Programa programa, boolean valor) {
        boolean validar = valor;

        if (programa.getUnidadeExecutora() == null || programa.getUnidadeExecutora().getId() == null) {
            addMsgError("O campo 'Unidade Executora' é obrigatório.");
            validar = false;
        }

        return validar;
    }

    public boolean validarLimitacaoGeografica(Programa programa, boolean valor) {
        boolean validar = valor;

        if (programa.getPossuiLimitacaoGeografica() == null) {
            addMsgError("O campo 'Limitação Geográfica' é obrigatório");
            validar = false;
        }

        return validar;
    }

    public boolean validarPotenciaisBeneficiarios(Programa programa, boolean valor) {
        boolean validar = valor;

        if (programa.getPossuiLimitacaoGeografica() != null && programa.getPossuiLimitacaoGeografica()) {
            if (programa.getPotenciaisBeneficiariosUf() == null || programa.getPotenciaisBeneficiariosUf().isEmpty()) {
                addMsgError("É necessário a escolha de pelo menos uma UF'");
                validar = false;
            }

        }

        if (programa.getPossuiLimitacaoMunicipalEspecifica() != null && programa.getPossuiLimitacaoMunicipalEspecifica()) {
            if (programa.getPotenciaisBeneficiariosUf() != null || !programa.getPotenciaisBeneficiariosUf().isEmpty()) {
                int i = 0;
                for (ProgramaPotencialBeneficiarioUf p : programa.getPotenciaisBeneficiariosUf()) {
                    if (p.getPotencialBeneficiarioMunicipios() != null && !p.getPotencialBeneficiarioMunicipios().isEmpty()) {
                        i++;
                    }
                }

                if (i == 0) {
                    addMsgError("É necessário a escolha de pelo menos um Município'");
                    validar = false;
                }
            }

        }

        return validar;
    }

    public boolean validarNomePersonalidadeJuridica(Programa programa, boolean valor) {
        boolean validar = valor;

        if (programa.getTipoPersonalidadeJuridica() == null) {
            addMsgError("O campo 'Personalidade Jurídica' é obrigatório.");
            validar = false;
        }

        return validar;
    }

    public boolean validarBensKits(Programa programa, boolean valor) {
        boolean validar = valor;

        if ((programa.getProgramaKits() == null || programa.getProgramaKits().isEmpty()) && (programa.getProgramaBens() == null || programa.getProgramaBens().isEmpty())) {
            addMsgError("Adicione ao menos 1 Bem ou 1 Kit.");
            validar = false;
        } else {
            if (programa.getProgramaKits() == null || !programa.getProgramaKits().isEmpty()) {
                List<ProgramaKit> listaKit = programa.getProgramaKits();
                int flag = 0;
                int qtdMaxima = 0;
                int qtdMaximaMaior = 0;
                for (ProgramaKit kit : listaKit) {
                    if (kit.getQuantidade() == null || kit.getQuantidade() < 1) {
                        flag++;
                    }
                    
                    if(kit.getQuantidadePorProposta() == null || kit.getQuantidadePorProposta() == 0)
                    {
                        qtdMaxima++;
                    }
                    
                    if(kit.getQuantidadePorProposta()!=null &&(kit.getQuantidadePorProposta()>kit.getQuantidade()))
                    {
                        qtdMaximaMaior ++;
                    }
                }

                if (flag > 0) {
                    addMsgError("Existe Kit adicionado a lista com a QUANTIDADE TOTAL '0', informe um valor para este campo.");
                    validar = false;
                }
                else
                {
                    if(qtdMaxima > 0)
                    {
                        addMsgError("Existe Kit adicionado a lista com a QUANTIDADE MÁXIMA POR PROPOSTA '0', informe um valor para estes campos.");
                        validar = false;
                    }else{
                        if(qtdMaximaMaior > 0)
                        {
                            addMsgError("Existe Kit com a QUANTIDADE MÁXIMA POR PROPOSTA maior do que a QUANTIDADE TOTAL.");
                            validar = false;
                        }
                    }
                }  
            }

            if (programa.getProgramaBens() == null || !programa.getProgramaBens().isEmpty()) {
                List<ProgramaBem> listaBem = programa.getProgramaBens();
                int flag = 0;
                int qtdMaxima = 0;
                int qtdMaximaMaior = 0;
                for (ProgramaBem bem : listaBem) {
                    if (bem.getQuantidade() == null || bem.getQuantidade() < 1) {
                        flag++;
                    }
                    
                    if(bem.getQuantidadePorProposta() == null || bem.getQuantidadePorProposta() == 0)
                    {
                        qtdMaxima++;
                    }
                    
                    if(bem.getQuantidadePorProposta()!=null &&(bem.getQuantidadePorProposta()>bem.getQuantidade()))
                    {
                        qtdMaximaMaior ++;
                    }
                }

                if (flag > 0) {
                    addMsgError("Existe Bem adicionado a lista com a QUANTIDADE TOTAL '0', informe um valor para este campo.");
                    validar = false;
                }
                else
                {
                    if(qtdMaxima > 0)
                    {
                        addMsgError("Existe Bem adicionado a lista com a QUANTIDADE MÁXIMA POR PROPOSTA '0', informe um valor para estes campos.");
                        validar = false;
                    }
                    else
                    {
                        if(qtdMaximaMaior > 0)
                        {
                            addMsgError("Existe Bem com a QUANTIDADE MÁXIMA POR PROPOSTA maior do que a QUANTIDADE TOTAL.");
                            validar = false;
                        }
                    }
                }
            }
        }

        return validar;
    }

    public boolean validarNumeroProcessoSei(Programa programa, boolean valor) {
        boolean validar = valor;

        if (programa.getNumeroProcessoSEI() == null || "".equalsIgnoreCase(programa.getNumeroProcessoSEI())) {
            addMsgError("O campo 'Número do Processo SEI' é obrigatório.");
            validar = false;
        } else {
            if (programa.getNumeroProcessoSEI().length() < 17) {
                addMsgError("O campo 'Número do Processo SEI' deverá conter 17 caracteres númericos.");
                validar = false;
            }
        }

        return validar;
    }

    public boolean validarCriterioElegibilidade(Programa programa, boolean valor) {
        boolean validar = valor;

        if (programa.getCriteriosElegibilidade() == null || programa.getCriteriosElegibilidade().isEmpty()) {
            addMsgError("Adicione ao menos 1 Critério de Elegibilidade e de Avaliação de Propostas.");
            validar = false;
        }

        return validar;
    }

    public boolean validarCriterioAcompanhamento(Programa programa, boolean valor) {
        boolean validar = valor;

        if (programa.getCriteriosAcompanhamento() == null || programa.getCriteriosAcompanhamento().isEmpty()) {
            addMsgError("Adicione ao menos 1 Critério de Acompanhamento.");
            validar = false;
        }

        return validar;
    }

    private String limparNumeroSei(String valor) {
        if (valor == null || "".equalsIgnoreCase(valor)) {
            return "";
        }
        String value = valor;
        value = value.replace(".", "");
        value = value.replace("/", "");
        value = value.replace("-", "");
        return value;
    }

    public void actionDropDownOrgao(DropDownChoice drop) {
        drop.add(new AjaxFormComponentUpdatingBehavior("onchange") {
            private static final long serialVersionUID = 1L;

            @Override
            protected void onUpdate(AjaxRequestTarget target) {

                if (orgao == null || orgao.getId() == null) {
                    unidadeExecutora = new UnidadeExecutora();
                    form.getModelObject().setUnidadeExecutora(unidadeExecutora);
                }

                panelOrgaoExecutor.addOrReplace(getDropDownUnidadeExecutora());
                target.add(panelOrgaoExecutor);
            }
        });
    }

    public void actionDropDownUnidadeExecutora(DropDownChoice drop) {
        drop.add(new AjaxFormComponentUpdatingBehavior("onchange") {
            private static final long serialVersionUID = 1L;

            @Override
            protected void onUpdate(AjaxRequestTarget target) {
                form.getModelObject().setUnidadeExecutora(unidadeExecutora);
            }
        });
    }

    public Form<Programa> getForm() {
        return form;
    }

    public void setForm(Form<Programa> form) {
        this.form = form;
    }

    public RecursoFinanceiroPanel getRecursoFinanceiroPanel() {
        return recursoFinanceiroPanel;
    }

    public void setRecursoFinanceiroPanel(RecursoFinanceiroPanel recursoFinanceiroPanel) {
        this.recursoFinanceiroPanel = recursoFinanceiroPanel;
    }

    public Programa getEntity() {
        return entity;
    }

    public void setEntity(Programa entity) {
        this.entity = entity;
    }

    public boolean isReadOnly() {
        return readOnly;
    }

    public void setReadOnly(boolean readOnly) {
        this.readOnly = readOnly;
    }

    public PanelInformacoesGeraisPage getPanelInformacoesGerais() {
        return panelInformacoesGerais;
    }

    public void setPanelInformacoesGerais(PanelInformacoesGeraisPage panelInformacoesGerais) {
        this.panelInformacoesGerais = panelInformacoesGerais;
    }
}
