package br.gov.mj.side.web.view.programa.visualizacao;

import java.io.IOException;
import java.io.OutputStream;
import java.util.Collections;
import java.util.List;

import javax.inject.Inject;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Button;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.PropertyListView;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.request.handler.resource.ResourceStreamRequestHandler;
import org.apache.wicket.util.convert.IConverter;
import org.apache.wicket.util.resource.AbstractResourceStreamWriter;

import br.gov.mj.infra.wicket.component.ComponentFactory;
import br.gov.mj.side.entidades.programa.Programa;
import br.gov.mj.side.entidades.programa.inscricao.analise.ListaAvaliacaoPublicado;
import br.gov.mj.side.entidades.programa.inscricao.analise.ListaElegibilidadePublicado;
import br.gov.mj.side.web.service.ProgramaService;
import br.gov.mj.side.web.view.components.converters.LocalDateTimeConverter;

public class ProgramaAnexosPublicadosPanel extends Panel {
    private static final long serialVersionUID = 1L;

    @Inject
    private ProgramaService programaService;

    @Inject
    private ComponentFactory componentFactory;

    private Programa programa;

    public ProgramaAnexosPublicadosPanel(String id, Programa programa) {
        super(id);
        this.programa = programa;
        add(newListViewListaElegibilidadePublicado());
        add(newListViewListaAvalicaoPublicado());
    }

    private PropertyListView<ListaElegibilidadePublicado> newListViewListaElegibilidadePublicado() {

        return new PropertyListView<ListaElegibilidadePublicado>("listaElegibilidade", getLista()) {
            private static final long serialVersionUID = 1L;

            @Override
            protected void populateItem(ListItem<ListaElegibilidadePublicado> item) {
                item.add(new Label("nomeArquivo"));
                item.add(new Label("dataCadastro") {
                    private static final long serialVersionUID = 1L;

                    @SuppressWarnings("unchecked")
                    @Override
                    public <C> IConverter<C> getConverter(Class<C> type) {
                        return (IConverter<C>) new LocalDateTimeConverter();
                    }
                });
                item.add(new Label("tamanhoArquivoEmMB"));
                item.add(newButtonDownload(item));

            }

            private Button newButtonDownload(ListItem<ListaElegibilidadePublicado> item) {
                Button btn = componentFactory.newButton("btnDownload", () -> downloadElegibilidade(item));
                btn.setDefaultFormProcessing(false);
                return btn;
            }
        };
    }

    private PropertyListView<ListaAvaliacaoPublicado> newListViewListaAvalicaoPublicado() {

        return new PropertyListView<ListaAvaliacaoPublicado>("listaAvaliacao", getListaAvaliacaoPublicado()) {
            private static final long serialVersionUID = 1L;

            @Override
            protected void populateItem(ListItem<ListaAvaliacaoPublicado> item) {
                item.add(new Label("nomeArquivo"));
                item.add(new Label("dataCadastro") {
                    private static final long serialVersionUID = 1L;

                    @SuppressWarnings("unchecked")
                    @Override
                    public <C> IConverter<C> getConverter(Class<C> type) {
                        return (IConverter<C>) new LocalDateTimeConverter();
                    }
                });
                item.add(new Label("tamanhoArquivoEmMB"));
                item.add(newButtonDownload(item));

            }

            private Button newButtonDownload(ListItem<ListaAvaliacaoPublicado> item) {
                Button btn = componentFactory.newButton("btnDownload", () -> downloadAvaliacao(item));
                btn.setDefaultFormProcessing(false);
                return btn;
            }
        };
    }

    private List<ListaAvaliacaoPublicado> getListaAvaliacaoPublicado() {
        if(programa.getId()!= null){
            return programaService.buscarListaAvaliacaoPublicado(programa);
        }
        return Collections.emptyList();
    }

    private List<ListaElegibilidadePublicado> getLista() {
        if (programa.getId() == null) {
            return Collections.emptyList();
        }
        return programaService.buscarListaElegibilidadePublicado(programa);
    }

    private void downloadElegibilidade(ListItem<ListaElegibilidadePublicado> item) {
        ListaElegibilidadePublicado lista = item.getModelObject();
        AbstractResourceStreamWriter rstream = new AbstractResourceStreamWriter() {
            private static final long serialVersionUID = 1L;

            @Override
            public void write(OutputStream output) throws IOException {
                output.write(lista.getConteudo());
            }
        };
        ResourceStreamRequestHandler handler = new ResourceStreamRequestHandler(rstream, lista.getNomeArquivo());
        getRequestCycle().scheduleRequestHandlerAfterCurrent(handler);
    }

    private void downloadAvaliacao(ListItem<ListaAvaliacaoPublicado> item) {
        ListaAvaliacaoPublicado lista = item.getModelObject();
        AbstractResourceStreamWriter rstream = new AbstractResourceStreamWriter() {
            private static final long serialVersionUID = 1L;

            @Override
            public void write(OutputStream output) throws IOException {
                output.write(lista.getConteudo());
            }
        };
        ResourceStreamRequestHandler handler = new ResourceStreamRequestHandler(rstream, lista.getNomeArquivo());
        getRequestCycle().scheduleRequestHandlerAfterCurrent(handler);
    }
}
