package br.gov.mj.side.web.view.programa.visualizacao;

import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.CompoundPropertyModel;

import br.gov.mj.side.entidades.programa.Programa;
import br.gov.mj.side.web.view.programa.anexo.PanelAnexoPage;

/**
 * Panel para visualizar as informações de um Programa
 * 
 * @author william.barreto
 */

public class ProgramaPanel extends Panel {
    private static final long serialVersionUID = 1L;

    private Programa programa;

    private ProgramaInformacoesGeraisPanel informacoesGeraisPanel;
    private ProgramaRecursoFinanceiroPanel recursoFinanceiroPanel;
    private ProgramaOrgaoExecutorPanel orgaoExecutorPanel;
    private ProgramaPotenciaisBeneficiariosPanel potenciaisBeneficiariosPanel;
    private ProgramaBemKitPanel bemKitPanel;
    private ProgramaCriterioElegebilidadePanel criterioElegibilidadePanel;
    private ProgramaCriterioAcompanhamentoPanel criterioAcompanhamentoPanel;
    private ProgramaCriterioAvaliacaoPanel criterioAvaliacaoPanel;
    private PanelAnexoPage anexoPanel;
    private ProgramaAnexosPublicadosPanel programaAnexosPublicadosPanel;

    public ProgramaPanel(String id, Programa entity) {
        super(id, new CompoundPropertyModel<Programa>(entity));
        this.programa = entity;
        initComponents();
    }

    private void initComponents() {
        informacoesGeraisPanel = new ProgramaInformacoesGeraisPanel("informacoesGeraisPanel");
        add(informacoesGeraisPanel);

        recursoFinanceiroPanel = new ProgramaRecursoFinanceiroPanel("recursoFinanceiroPanel", programa);
        add(recursoFinanceiroPanel);

        orgaoExecutorPanel = new ProgramaOrgaoExecutorPanel("orgaoExecutorPanel");
        add(orgaoExecutorPanel);

        potenciaisBeneficiariosPanel = new ProgramaPotenciaisBeneficiariosPanel("potenciaisBeneficiariosPanel", programa);
        add(potenciaisBeneficiariosPanel);

        bemKitPanel = new ProgramaBemKitPanel("bemKitPanel", programa);
        add(bemKitPanel);

        criterioElegibilidadePanel = new ProgramaCriterioElegebilidadePanel("criterioElegibilidadePanel", programa);
        add(criterioElegibilidadePanel);

        criterioAcompanhamentoPanel = new ProgramaCriterioAcompanhamentoPanel("criterioAcompanhamentoPanel", programa);
        add(criterioAcompanhamentoPanel);
        
        criterioAvaliacaoPanel = new ProgramaCriterioAvaliacaoPanel("criterioAvaliacaoPanel", programa);
        add(criterioAvaliacaoPanel);
        
        
        anexoPanel = new PanelAnexoPage("anexoPanel", programa, false);
        add(anexoPanel);

        programaAnexosPublicadosPanel = new ProgramaAnexosPublicadosPanel("programaAnexosPublicadosPanel", programa);
        add(programaAnexosPublicadosPanel);
    }

}
