package br.gov.mj.side.web.view.propostas;

import java.util.Arrays;
import java.util.Iterator;

import javax.inject.Inject;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.ajax.markup.html.form.AjaxSubmitLink;
import org.apache.wicket.authroles.authorization.strategies.role.annotations.AuthorizeInstantiation;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.OrderByBorder;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Button;
import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.markup.repeater.data.DataView;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.validation.validator.StringValidator;

import br.gov.mj.apoio.entidades.Orgao;
import br.gov.mj.apoio.entidades.Uf;
import br.gov.mj.infra.negocio.persistencia.EnumOrder;
import br.gov.mj.infra.wicket.component.ComponentFactory;
import br.gov.mj.infra.wicket.component.InfraDropDownChoice;
import br.gov.mj.infra.wicket.component.pagination.InfraAjaxPagingNavigator;
import br.gov.mj.infra.wicket.model.LambdaModel;
import br.gov.mj.side.entidades.enums.EnumStatusInscricao;
import br.gov.mj.side.entidades.programa.inscricao.InscricaoPrograma;
import br.gov.mj.side.web.dto.PropostaPesquisaDto;
import br.gov.mj.side.web.service.InscricaoProgramaService;
import br.gov.mj.side.web.service.OrgaoService;
import br.gov.mj.side.web.service.UfService;
import br.gov.mj.side.web.util.CnpjUtil;
import br.gov.mj.side.web.util.Constants;
import br.gov.mj.side.web.view.components.validators.YearIntegerValidator;
import br.gov.mj.side.web.view.kit.KitPagePesquisa;
import br.gov.mj.side.web.view.template.TemplatePage;

@AuthorizeInstantiation({ PropostaPesquisaPage.ROLE_MANTER_ANALISE_VISUALIZAR, PropostaPesquisaPage.ROLE_MANTER_ANALISE_ELEGIBILIDADE, PropostaPesquisaPage.ROLE_MANTER_ANALISE_AVALIACAO })
public class PropostaPesquisaPage extends TemplatePage {
    private static final long serialVersionUID = 1L;

    public static final String ROLE_MANTER_ANALISE_VISUALIZAR = "manter_analise:visualizar";
    public static final String ROLE_MANTER_ANALISE_ELEGIBILIDADE = "manter_analise:analisar_elegibilidade";
    public static final String ROLE_MANTER_ANALISE_AVALIACAO = "manter_analise:analisar_avaliacao";

    private PanelComponentes panelComponentes;
    private PanelBotoes panelBotoes;
    private PanelResultados panelResultados;
    
    private Form<PropostaPesquisaDto> form;
    
    private DataView<InscricaoPrograma> dataView;
    
    private Integer itensPorPagina = Constants.ITEMS_PER_PAGE_PAGINATION;
    private Boolean mostrarDataView=false;
    private InscricoesProvider inscricoesProvider;

    @Inject
    private ComponentFactory componentFactory;

    @Inject
    private UfService ufService;

    @Inject
    private OrgaoService orgaoService;
    
    @Inject
    private InscricaoProgramaService inscricaoService;

    public PropostaPesquisaPage(final PageParameters pageParameters) {
        super(pageParameters);
        // criarBreadcrumb();

        setTitulo("Pesquisar Propostas");
        initComponents();
    }

    private class PanelComponentes extends WebMarkupContainer {
        private static final long serialVersionUID = 1L;

        public PanelComponentes(String id) {
            super(id);
            setOutputMarkupId(true);

            add(newTextFieldCodigoPrograma()); // codigoIdentificadorProgramaPublicado
            add(newTextFieldNome()); // inscricao.programa.nomePrograma
            add(newTextFieldAno()); // inscricao.programa.anoPrograma
            add(newDropDownOrgao()); // inscricao.programa.unidadeExecutora.orgao
            add(newTextFieldProponente()); // inscricao.pessoaEntidade.entidade.nomeEntidade
            add(newTextFieldCnpj()); // inscricao.pessoaEntidade.entidade.numeroCnpj
            add(newDropDownUf()); // inscricao.pessoaEntidade.entidade.municipio.uf
            add(newTextNumeroInscricao()); //inscricao.id
        }
    }

    private class PanelBotoes extends WebMarkupContainer {
        private static final long serialVersionUID = 1L;

        public PanelBotoes(String id) {
            super(id);
            setOutputMarkupId(true);

            add(newButtonPesquisar()); // btnPesquisar
            add(newButtonLimpar()); // btnLimpar
        }
    }

    private class PanelResultados extends WebMarkupContainer {
        private static final long serialVersionUID = 1L;

        public PanelResultados(String id) {
            super(id);
            setOutputMarkupId(true);
            
            inscricoesProvider=new InscricoesProvider();
            
            add(newDataViewInscricoes(inscricoesProvider)); // dataPropostas
            add(newDropItensPorPagina()); //itensPorPagina
            
            add(new InfraAjaxPagingNavigator("pagination", dataView));
            add(new OrderByBorder<String>("orderByCodigo", "programa.identificadorProgramaPublicado", inscricoesProvider));
            add(new OrderByBorder<String>("orderByProponente", "pessoaEntidade.entidade.nomeEntidade", inscricoesProvider));
            add(new OrderByBorder<String>("orderByCnpj", "pessoaEntidade.entidade.numeroCnpj", inscricoesProvider));
            add(new OrderByBorder<String>("orderBySituacao", "statusInscricao", inscricoesProvider));
            add(new OrderByBorder<String>("orderByIdInscricao", "id", inscricoesProvider));
        }
    }

    private void initComponents() {
        form = componentFactory.newForm("form", new PropostaPesquisaDto());
        add(form);

        panelComponentes = new PanelComponentes("panelComponentes");
        panelBotoes = new PanelBotoes("panelBotoes");
        panelResultados = new PanelResultados("panelResultados");
        panelResultados.setVisible(false);

        form.add(panelComponentes);
        form.add(panelBotoes);
        form.add(panelResultados);
    }

    // COMPONENTES

    private TextField<String> newTextFieldCodigoPrograma() {
        TextField<String> textField = componentFactory.newTextField("inscricao.programa.codigoIdentificadorProgramaPublicadoTemp", "Código", false, null);
        textField.add(StringValidator.maximumLength(30));
        return textField;

    }

    private TextField<String> newTextFieldNome() {
        return componentFactory.newTextField("inscricao.programa.nomePrograma", "Nome", false, null);
    }

    private TextField<Integer> newTextFieldAno() {
        return componentFactory.newTextField("inscricao.programa.anoPrograma", "Ano", false, null, new YearIntegerValidator());

    }

    private InfraDropDownChoice<Orgao> newDropDownOrgao() {
        InfraDropDownChoice<Orgao> dropDownChoice = componentFactory.newDropDownChoice("inscricao.programa.unidadeExecutora.orgao", "Órgão", false, "id", "codigoOrgaoSigla", null, orgaoService.buscarTodos(), null);
        dropDownChoice.setNullValid(true);

        dropDownChoice.add(new AjaxFormComponentUpdatingBehavior("onchange") {
            private static final long serialVersionUID = 1L;

            @Override
            protected void onUpdate(AjaxRequestTarget target) {
                // Setar no model
            }
        });
        return dropDownChoice;
    }

    private TextField<String> newTextFieldProponente() {
        TextField<String> textFieldProponente = componentFactory.newTextField("inscricao.pessoaEntidade.entidade.nomeEntidade", "Proponente", false, null);
        textFieldProponente.add(StringValidator.maximumLength(30));
        return textFieldProponente;
    }

    private TextField<String> newTextFieldCnpj() {
        TextField<String> textFieldCnpj = componentFactory.newTextField("inscricao.pessoaEntidade.entidade.numeroCnpj", "CNPJ", false, null);
        textFieldCnpj.add(StringValidator.maximumLength(30));
        return textFieldCnpj;
    }

    private InfraDropDownChoice<Uf> newDropDownUf() {
        InfraDropDownChoice<Uf> dropDownUf = componentFactory.newDropDownChoice("inscricao.pessoaEntidade.entidade.municipio.uf", "Estado", false, "id", "nomeSigla", null, ufService.buscarTodos(), null);
        dropDownUf.setNullValid(true);
        return dropDownUf;
    }
    
    private TextField<Integer> newTextNumeroInscricao() {
        TextField<Integer> textFieldNumeroInscricao = componentFactory.newTextField("inscricao.id", "Nº da Proposta", false, null);
        textFieldNumeroInscricao.setOutputMarkupId(true);
        return textFieldNumeroInscricao;
    }
    
    
    private DataView<InscricaoPrograma> newDataViewInscricoes(InscricoesProvider ip) {
        dataView = new DataView<InscricaoPrograma>("dataPropostas", ip) {
            private static final long serialVersionUID = 1L;

            @Override
            protected void populateItem(Item<InscricaoPrograma> item) {
                item.add(new Label("codigoPrograma",item.getModelObject().getPrograma().getCodigoIdentificadorProgramaPublicado()));
                
                String inscricaoId=item.getModelObject().getId().toString();
                StringBuilder sb=new StringBuilder(inscricaoId);
                while(sb.length()<7)
                {
                    sb.insert(0, '0');
                }
                
                
                item.add(new Label("codigoInscricao",sb.toString()));
                item.add(new Label("nomeEntidade",item.getModelObject().getPessoaEntidade().getEntidade().getNomeEntidade()));
                item.add(new Label("cnpjEntidade",CnpjUtil.imprimeCNPJ(item.getModelObject().getPessoaEntidade().getEntidade().getNumeroCnpj())));
                item.add(new Label("situacaoEntidade",item.getModelObject().getStatusInscricao().getDescricao()));
                item.add(newButtonVisualizar(item)); // btnVisualizar
                item.add(newButtonAnalisar(item)); // btnAnalisar   
            }
        };
        dataView.setItemsPerPage(Constants.ITEMS_PER_PAGE_PAGINATION);
        return dataView;
    }
    
    
    private DropDownChoice<Integer> newDropItensPorPagina() {
        DropDownChoice<Integer> dropDownChoice = new DropDownChoice<Integer>("itensPorPagina", new LambdaModel<Integer>(this::getItensPorPagina, this::setItensPorPagina), Arrays.asList(5, 10, 15, 20, 25, 30));
        dropDownChoice.add(new AjaxFormComponentUpdatingBehavior("onchange") {
            private static final long serialVersionUID = 1L;

            @Override
            protected void onUpdate(AjaxRequestTarget target) {
                dataView.setItemsPerPage(getItensPorPagina());
                target.add(panelResultados);
            };
        });
        return dropDownChoice;
    }

    // BOTÕES
    
    private Button newButtonPesquisar() {
        return componentFactory.newButton("btnPesquisar", () -> pesquisar());
    }
    
    private void pesquisar()
    {
        if(form.getModelObject().getInscricao()==null)
        {
            InscricaoPrograma inscricao=new InscricaoPrograma();
            form.getModelObject().setInscricao(inscricao);
        }
        panelResultados.setVisible(true);
    }
    public AjaxSubmitLink newButtonPesquisare() {
        AjaxSubmitLink button = new AjaxSubmitLink("btnPesquisar", form) {
            private static final long serialVersionUID = 1L;

            @Override
            public void onSubmit(AjaxRequestTarget target, Form forme) {
                
                if(form.getModelObject().getInscricao()==null)
                {
                    InscricaoPrograma inscricao=new InscricaoPrograma();
                    form.getModelObject().setInscricao(inscricao);
                }
                panelResultados.setVisible(true);
                
                panelResultados.addOrReplace(newDataViewInscricoes(inscricoesProvider));
                panelResultados.addOrReplace(new InfraAjaxPagingNavigator("pagination", dataView));
                target.add(panelResultados);
            }
        };
        authorize(button, RENDER, KitPagePesquisa.ROLE_MANTER_KIT_VISUALIZAR);
        return button;
    }

    public AjaxSubmitLink newButtonLimpar() {
        AjaxSubmitLink buttonCancelarEdicao = new AjaxSubmitLink("btnLimpar", form) {
            private static final long serialVersionUID = 1L;

            @Override
            public void onSubmit(AjaxRequestTarget target, Form formulario) {

                form.setModelObject(new PropostaPesquisaDto());
                
                panelResultados.setVisible(false);
                target.add(panelResultados);
                target.add(panelComponentes);
            }
        };
        return buttonCancelarEdicao;
    }

    private Button newButtonVisualizar(Item<InscricaoPrograma> item) {
        return componentFactory.newButton("btnVisualizar", () -> visualizar(item));
    }

    private Button newButtonAnalisar(Item<InscricaoPrograma> item) {
        Button btnAnalisar = componentFactory.newButton("btnAnalisar", () -> analisar(item));
        btnAnalisar.setVisible(mostrarBotaoAnalisar(item));
        return btnAnalisar;
    }

    // PROVIDER

    private class InscricoesProvider extends SortableDataProvider<InscricaoPrograma, String> {
        private static final long serialVersionUID = 1L;

        public InscricoesProvider() {
            setSort("programa.nomePrograma", SortOrder.ASCENDING);
        }

        @Override
        public Iterator<InscricaoPrograma> iterator(long first, long size) {
            return inscricaoService.buscarPaginado(form.getModelObject().getInscricao(), (int) first, (int) size, getSort().isAscending() ? EnumOrder.ASC : EnumOrder.DESC, getSort().getProperty()).iterator();
        }

        @Override
        public long size() {
            return inscricaoService.buscarSemPaginacao(form.getModelObject().getInscricao()).size();
        }

        @Override
        public IModel<InscricaoPrograma> model(InscricaoPrograma object) {
            return new CompoundPropertyModel<InscricaoPrograma>(object);
        }
    }

    // AÇÕES

    private boolean mostrarBotaoAnalisar(Item<InscricaoPrograma> item)
    {
        EnumStatusInscricao statusInscricao=item.getModelObject().getStatusInscricao();
        if(statusInscricao == EnumStatusInscricao.EM_ELABORACAO || statusInscricao == EnumStatusInscricao.FINALIZADA)
        {
            return false;
        }
        else
        {
            return true;
        }
    }
    
    private void pesquisar(AjaxRequestTarget target) {
        System.out.println("Realiza a pesquisa");
        
    }

    private void visualizar(Item<InscricaoPrograma> item) {
         setResponsePage(new PropostaPage(getPageParameters(),item.getModelObject(),this,true));
    }

    private void analisar(Item<InscricaoPrograma> item) {
        setResponsePage(new PropostaPage(getPageParameters(),item.getModelObject(),this,false));
    }

    public Integer getItensPorPagina() {
        return itensPorPagina;
    }

    public void setItensPorPagina(Integer itensPorPagina) {
        this.itensPorPagina = itensPorPagina;
    }

    public Boolean getMostrarDataView() {
        return mostrarDataView;
    }

    public void setMostrarDataView(Boolean mostrarDataView) {
        this.mostrarDataView = mostrarDataView;
    }

}
